@@ -13,7 +13,7 @@ use Net::LDAP;
 require "modules/tool.pl";
 
 my %conf;
-if (($_=ow::tool::find_configfile('etc/auth_ldap.conf', 'etc/auth_ldap.conf.default')) ne '') {
+if (($_=ow::tool::find_configfile('etc/auth_ldap.conf', 'etc/defaults/auth_ldap.conf')) ne '') {
    my ($ret, $err)=ow::tool::load_configfile($_, \%conf);
    die $err if ($ret<0);
 } else {
@@ -137,7 +137,7 @@ sub change_userpassword {
    $ldap->bind (dn=>"$cn, $dc1, $dc2", password =>$pwd) or  return(-3, "LDAP error $@");
 
    my $mesg = $ldap->modify (
-                            dn      => "uid=$user, ou=People, $dc1, $dc2",
+                            dn      => "uid=$user, $ou, $dc1, $dc2",
                             replace => {'userPassword'=>$encrypted}
                             );
    undef($ldap);
