@@ -4,7 +4,7 @@ use strict;
 # auth_pam.pl - authenticate user with PAM
 #
 # 2002/08/01 webmaster.AT.pkgmaster.com
-#            add check for nologin, validshell, cobaltuser 
+#            add check for nologin, validshell, cobaltuser
 #            based on work from Trevor.Paquette.AT.TeraGo.ca
 # 2001/10/05 tung.AT.turtle.ee.ncku.edu.tw
 #
@@ -21,12 +21,12 @@ require "modules/filelock.pl";
 require "modules/tool.pl";
 
 my %conf;
-if (($_=ow::tool::find_configfile('etc/auth_pam.conf', 'etc/auth_pam.conf.default')) ne '') {
+if (($_=ow::tool::find_configfile('etc/auth_pam.conf', 'etc/defaults/auth_pam.conf')) ne '') {
    my ($ret, $err)=ow::tool::load_configfile($_, \%conf);
    die $err if ($ret<0);
 }
 
-my $servicename = $conf{'servicename'} || "openwebmail";
+my $servicename = $conf{'servicename'} || "login";
 my $passwdfile_plaintext = $conf{'passwdfile_plaintext'} || "/etc/passwd";
 
 my $check_nologin = $conf{'check_nologin'} || 'no';
@@ -129,8 +129,8 @@ sub check_userpassword {
       return @res;
    }
 
-   # disable SIG CHLD since authsys in PAM may fork process
-   local $SIG{CHLD}; undef $SIG{CHLD};
+   # disable $SIG{CHLD} temporary since authsys in PAM may fork process
+   # local $SIG{CHLD}; undef $SIG{CHLD};	# already done in auth.pl
 
    my ($pamh, $ret, $errmsg);
    if ( ref($pamh = new Authen::PAM($servicename, $pam_user, \&checkpwd_conv_func)) ) {
@@ -179,6 +179,12 @@ sub change_userpassword {
    ($r_config, $pam_user, $pam_password, $pam_newpassword)=@_;
    return (-2, "User or password is null") if ($pam_user eq '' || $pam_password eq '' || $pam_newpassword eq '');
 
+   # the PAM of underlying authentication may forget to check old password
+   # before changing password, so we check old password explicitly here
+   if ((check_userpassword($r_config, $pam_user, $pam_password))[0] != 0) {
+      return (-4, "Password incorrect");
+   }
+
    local $pam_convstate=0;	# localized global to make reentry safe
    sub changepwd_conv_func {
       my @res;
@@ -206,7 +212,7 @@ sub change_userpassword {
    }
 
    # disable SIG CHLD since authsys in PAM may fork process
-   local $SIG{CHLD}; undef $SIG{CHLD};
+   # local $SIG{CHLD}; undef $SIG{CHLD};	# already done in auth.pl
 
    my ($pamh, $ret, $errmsg);
    if (ref($pamh = new Authen::PAM($servicename, $pam_user, \&changepwd_conv_func)) ) {
@@ -226,7 +232,7 @@ sub change_userpassword {
 
 ########## misc support routine ##################################
 
-# this routie is slower than system getpwnam() but can work with file
+# this routine is slower than system getpwnam() but can work with file
 # other than /etc/passwd. ps: it always return '*' for passwd field.
 sub getpwnam_file {
    my ($user, $passwdfile_plaintext)=@_;
