@@ -13,7 +13,7 @@ require "modules/filelock.pl";
 require "modules/tool.pl";
 
 my %conf;
-if (($_=ow::tool::find_configfile('etc/auth_vdomain.conf', 'etc/auth_vdomain.conf.default')) ne '') {
+if (($_=ow::tool::find_configfile('etc/auth_vdomain.conf', 'etc/defaults/auth_vdomain.conf')) ne '') {
    my ($ret, $err)=ow::tool::load_configfile($_, \%conf);
    die $err if ($ret<0);
 }
@@ -59,7 +59,7 @@ sub get_userinfo {
    if ( ${$r_config}{'use_syshomedir'} && -d $homedir) {
       # mkdir domainhome so openwebmail.pl can create user homedir under this domainhome
       if (! -d $domainhome) {
-         my $mailgid=getgrnam('mail');
+         my $mailgid=getgrnam('_mail');
          $domainhome = ow::tool::untaint($domainhome);
          mkdir($domainhome, 0750);
          return(-3, "Couldn't create domain homedir $domainhome") if (! -d $domainhome);
@@ -193,21 +193,22 @@ sub change_userpassword {
       return (-3, "Unable to match entry for modification");
    }
 
-   open(TMP, ">$pwdfile.tmp.$$") or goto authsys_error;
+   my $tmpfile=ow::tool::untaint("$pwdfile.tmp.$$.".rand());
+   open(TMP, ">$tmpfile") or goto authsys_error;
    print TMP $content or goto authsys_error;
    close(TMP) or goto authsys_error;
 
    # automic update passwd by rename
    my ($fmode, $fuid, $fgid) = (stat($pwdfile))[2,4,5];
-   chown($fuid, $fgid, "$pwdfile.tmp.$$");
-   chmod($fmode, "$pwdfile.tmp.$$");
-   rename("$pwdfile.tmp.$$", $pwdfile) or goto authsys_error;
+   chown($fuid, $fgid, $tmpfile);
+   chmod($fmode, $tmpfile);
+   rename($tmpfile, $pwdfile) or goto authsys_error;
 
    ow::filelock::lock($pwdfile, LOCK_UN);
    return (0, '');
 
 authsys_error:
-   unlink("$pwdfile.tmp.$$");
+   unlink($tmpfile);
    ow::filelock::lock($pwdfile, LOCK_UN);
    return (-3, "Unable to write $pwdfile");
 }
