@@ -55,7 +55,7 @@ HTMLArea.I18N = {
 		indent:         " ",
 		forecolor:      "  ",
 		hilitecolor:    "  ",
-		horizontalrule: " ",
+		inserthorizontalrule: " ",
 		createlink:     " ",
 		insertimage:    " ",
 		insertsound:    " ",
