@@ -55,7 +55,7 @@ HTMLArea.I18N = {
 		indent:         "Aumentar Indentao",
 		forecolor:      "Cor da Fonte",
 		hilitecolor:    "Cor do Fundo",
-		horizontalrule: "Rgua Horizontal",
+		inserthorizontalrule: "Rgua Horizontal",
 		createlink:     "Inserir link",
 		insertimage:    "Inserir Imagem",
 		insertsound:    "Inserir Som",
