@@ -55,7 +55,7 @@ HTMLArea.I18N = {
 		indent:         "Zvi odsadenie",
 		forecolor:      "Farba psma",
 		hilitecolor:    "Farba pozadia",
-		horizontalrule: "Horizontlna iara",
+		inserthorizontalrule: "Horizontlna iara",
 		createlink:     "Vloi webov odkaz",
 		insertimage:    "Vloi obrzok",
 		insertsound:    "Vloi zvuk",
