@@ -55,7 +55,7 @@ HTMLArea.I18N = {
 		indent:         "",
 		forecolor:      "ɫ",
 		hilitecolor:    "ɫ",
-		horizontalrule: "ˮƽָ",
+		inserthorizontalrule: "ˮƽָ",
 		createlink:     "볬",
 		insertimage:    "ͼƬ",
 		insertsound:    "",
