@@ -19,7 +19,7 @@ use vars qw($dbm_ext $dbmopen_ext $dbmopen_haslock);
 use vars qw($dbm_errno $dbm_errmsg $dbm_warning);
 
 my %conf;
-if (($_=ow::tool::find_configfile('etc/dbm.conf', 'etc/dbm.conf.default')) ne '') {
+if (($_=ow::tool::find_configfile('etc/dbm.conf', 'etc/defaults/dbm.conf')) ne '') {
    my ($ret, $err)=ow::tool::load_configfile($_, \%conf);
    die $err if ($ret<0);
 }
@@ -87,7 +87,7 @@ sub open {
          }
       }
 
-      last;	# default to exit the loop
+      last;	# default to leave the loop
    }
    ($dbm_errno, $dbm_errmsg)=(-4 , $openerror);
    return 0;
@@ -113,7 +113,7 @@ sub rename {
    ($dbm_errno, $dbm_errmsg, $dbm_warning)=(0, '', '');
 
    if ($dbm_ext eq '.dir' || $dbm_ext eq '.pag') {
-     return 1 if (rename("$olddb.dir", "$newdb.dir") && 
+     return 1 if (rename("$olddb.dir", "$newdb.dir") &&
                   rename("$olddb.pag", "$newdb.pag") );
    } else {
      return 1 if (rename("$olddb$dbm_ext", "$newdb$dbm_ext") );
@@ -126,7 +126,7 @@ sub chown {
    my ($uid, $gid, @dblist)=@_;
    ($dbm_errno, $dbm_errmsg, $dbm_warning)=(0, '', '');
    return 1 if (chown($uid, $gid, dblist2dbfiles(@dblist)));
-   ($dbm_errno, $dbm_errmsg)=(-1, $!); 
+   ($dbm_errno, $dbm_errmsg)=(-1, $!);
    return 0;
 }
 
@@ -148,16 +148,17 @@ sub unlink {
 sub guessoptions {
    my (%DB, @filelist, @delfiles);
    my ($dbm_ext, $dbmopen_ext, $dbmopen_haslock);
+   my $testdir=ow::tool::tmpname('dbmtest.tmpdir');
 
-   mkdir ("/tmp/dbmtest.$$", 0755);
+   mkdir ($testdir, 0755);
 
-   dbmopen(%DB, "/tmp/dbmtest.$$/test", 0600); dbmclose(%DB);
+   dbmopen(%DB, "$testdir/test", 0600); dbmclose(%DB);
    @delfiles=();
-   opendir(TESTDIR, "/tmp/dbmtest.$$");
+   opendir(TESTDIR, $testdir);
    while (defined(my $filename = readdir(TESTDIR))) {
       if ($filename!~/^\./ ) {
          push(@filelist, $filename);
-         push(@delfiles, ow::tool::untaint("/tmp/dbmtest.$$/$filename"));
+         push(@delfiles, ow::tool::untaint("$testdir/$filename"));
       }
    }
    closedir(TESTDIR);
@@ -171,11 +172,11 @@ sub guessoptions {
    }
 
    my $result;
-   ow::filelock::lock("/tmp/dbmtest.$$/test$dbm_ext", LOCK_EX);
+   ow::filelock::lock("$testdir/test$dbm_ext", LOCK_EX);
    eval {
       local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
       alarm 5;	# timeout 5 sec
-      $result = dbmopen(%DB, "/tmp/dbmtest.$$/test$dbmopen_ext", 0600);
+      $result = dbmopen(%DB, "$testdir/test$dbmopen_ext", 0600);
       dbmclose(%DB) if ($result);
       alarm 0;
    };
@@ -184,17 +185,17 @@ sub guessoptions {
    } else {
       $dbmopen_haslock=0;
    }
-   ow::filelock::lock("/tmp/dbmtest.$$/test$dbm_ext", LOCK_UN);
+   ow::filelock::lock("$testdir/test$dbm_ext", LOCK_UN);
 
    @delfiles=();
-   opendir(TESTDIR, "/tmp/dbmtest.$$");
+   opendir(TESTDIR, $testdir);
    while (defined(my $filename = readdir(TESTDIR))) {
-      push(@delfiles, ow::tool::untaint("/tmp/dbmtest.$$/$filename")) if ($filename!~/^\./ );
+      push(@delfiles, ow::tool::untaint("$testdir/$filename")) if ($filename!~/^\./ );
    }
    closedir(TESTDIR);
    unlink(@delfiles) if ($#delfiles>=0);
 
-   rmdir("/tmp/dbmtest.$$");
+   rmdir($testdir);
 
    return($dbm_ext, $dbmopen_ext, $dbmopen_haslock);
 }
@@ -204,7 +205,7 @@ sub guessoptions {
 use vars qw($_defaultdbtype);
 sub get_defaultdbtype {
    if ($_defaultdbtype eq '') {
-      my $t=ow::tool::untaint("/tmp/.dbmtest.$$");
+      my $t=ow::tool::tmpname('dbmtest.tmpfile');
       my %t; dbmopen(%t, "$t$dbmopen_ext", 0600); dbmclose(%t);
 
       $_defaultdbtype=get_dbtype("$t$dbm_ext");
@@ -215,7 +216,7 @@ sub get_defaultdbtype {
 }
 
 sub get_dbtype {
-   my $f=ow::tool::untaint("/tmp/.flist.$$");
+   my $f=ow::tool::tmpname('flist.tmpfile');
    open(F, ">$f"); print F "$_[0]\n"; close(F);	# pass arg through file for safety
 
    my $dbtype=`/usr/bin/file -f $f`; unlink($f);
