@@ -12,7 +12,7 @@ use Fcntl qw(:DEFAULT :flock);
 require "modules/tool.pl";
 
 my %conf;
-if (($_=ow::tool::find_configfile('etc/filelock.conf', 'etc/filelock.conf.default')) ne '') {
+if (($_=ow::tool::find_configfile('etc/filelock.conf', 'etc/defaults/filelock.conf')) ne '') {
    my ($ret, $err)=ow::tool::load_configfile($_, \%conf);
    die $err if ($ret<0);
 }
@@ -33,7 +33,7 @@ use vars qw(%opentable);
 # this should be called at the end of each request to free the file handles
 sub closeall {
    foreach (keys %opentable) {
-      close($opentable{$_}{fh}) if ( defined(fileno($opentable{$_}{fh})) );
+      close($opentable{$_}{fh}) if (defined fileno($opentable{$_}{fh}));
    }
    %opentable=();
 }
@@ -53,14 +53,14 @@ sub flock_lock {
       ($dev, $inode)=(stat($filename))[0,1];
       return 0 if ($dev eq '' || $inode eq '');
 
-      if (defined($opentable{"$dev-$inode"}) ) {
+      if (defined $opentable{"$dev-$inode"}) {
          $fh=$opentable{"$dev-$inode"}{fh};
          $retval=flock($fh, LOCK_UN);
          if ($retval) {
             $opentable{"$dev-$inode"}{n}--;
             if ($opentable{"$dev-$inode"}{n}==0) {
                delete($opentable{"$dev-$inode"});
-               close($fh) if ( defined(fileno($fh)) );
+               close($fh) if (defined fileno($fh));
             }
          }
       } else {
@@ -78,7 +78,7 @@ sub flock_lock {
    ($dev, $inode)=(stat($filename))[0,1];
    return 0 if ($dev eq '' || $inode eq '');
 
-   if (!defined($opentable{"$dev-$inode"}) ) {
+   if (!defined $opentable{"$dev-$inode"}) {
       $fh=do { local *FH };
       if (sysopen($fh, $filename, O_RDWR) ||	# try RDWR open first
           sysopen($fh, $filename, O_RDONLY) ) {	# then RDONLY for readonly file
@@ -124,6 +124,8 @@ sub dotfile_lock {
    my ($filename, $lockflag, $perm)=@_;
    return 1 unless ($lockflag & (LOCK_SH|LOCK_EX|LOCK_UN));
 
+   local $/; $/="\n";	# just prevent in case $/ is not "\n" when this routine is called
+
    $filename=ow::tool::untaint($filename);
    if ( (! -e $filename) && $lockflag ne LOCK_UN) {
       $perm=0600 if (!$perm);
@@ -178,7 +180,7 @@ sub dotfile_lock {
                   seek(L, 0, 0);
                   my $rec="READ:$count\n";
                   print L $rec;
-                  truncate(L, length($rec));
+                  truncate(L, ow::tool::untaint(length($rec)));
                   close(L);
                   $status=1;
                } else {
@@ -212,7 +214,7 @@ sub dotfile_lock {
                seek(L,0,0);
                my $rec="READ:$count\n";
                print L $rec;
-               truncate(L, length($rec));
+               truncate(L, ow::tool::untaint(length($rec)));
                $status=1;
             } else {
                $status=0;
