@@ -36,7 +36,9 @@ my @jsevents=('onAbort', 'onBlur', 'onChange', 'onClick', 'onDblClick',
               'onDragDrop', 'onError', 'onFocus', 'onKeyDown', 'onKeyPress',
               'onKeyUp', 'onLoad', 'onMouseDown', 'onMouseMove', 'onMouseOut',
               'onMouseOver', 'onMouseUp', 'onMove', 'onReset', 'onResize',
-              'onSelect', 'onSubmit', 'onUnload', 'window.open');
+              'onSelect', 'onSubmit', 'onUnload', 'window.open',
+              '@import', 'window.location', 'location.href',
+              'document.url', 'document.location', 'document.referrer');
 
 # this routine is used to add target=_blank to links in a html message
 # so clicking on it will open a new window
@@ -80,7 +82,7 @@ sub _frame2iframe {
 sub html4disablejs {
    my $html=$_[0];
    foreach my $event (@jsevents) {
-      $html=~s/$event/_$event/isg;
+      $html=~s/$event/x_$event/isg;
    }
    $html=~s/<script([^\<\>]*?)>/<disable_script$1>\n<!--\n/isg;
    $html=~s/<!--\s*<!--/<!--/isg;
@@ -217,13 +219,21 @@ sub _link2cid {
 # this routine chnage mailto: into webmail composemail function
 # to make it works with base directive, we use full url
 # to make it compatible with undecoded base64 block,
-# we put new url into a seperate line
+# we put new url into a separated line
 sub html4mailto {
    my ($html, $scripturl, $scriptparm)=@_;
-   $html =~ s/(=\s*"?)mailto:\s?([^\s]*?)\s?(\s|"?\s*\>)/$1\n$scripturl\?$scriptparm&amp;to=$2\n$3/ig;
+   $html =~ s/(=\s*"?)mailto:\s?([^'"\>]*?)("|'|\>)/_mailtoparm($scripturl,$scriptparm,$1,$2,$3)/egis;
    return($html);
 }
 
+# convert param in mailto for outlook to parm for owm
+sub _mailtoparm {
+   my ($scripturl, $scriptparm, $prefix, $parm, $suffix)=@_;
+   $parm=~s/&amp;/&/g;
+   $parm=~s/\?/&/;
+   $parm=~s/&/&amp;/g;
+   return "$prefix\n$scripturl?$scriptparm&amp;to=$parm\n$suffix";
+}
 
 sub html2table {	# for msg reading
    my $html=_htmlclean($_[0]);
