@@ -4,14 +4,14 @@
 #
 
 use vars qw($SCRIPT_DIR);
-if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1 }
+if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { local $1; $SCRIPT_DIR=$1 }
 if ($SCRIPT_DIR eq '' && open(F, '/var/www/conf/openwebmail/openwebmail_path.conf')) {
-   $_=<F>; close(F); if ( $_=~/^(\S*)/) { $SCRIPT_DIR=$1 }
+   $_=<F>; close(F); if ( $_=~/^(\S*)/) { local $1; $SCRIPT_DIR=$1 }
 }
 if ($SCRIPT_DIR eq '') { print "Content-type: text/html\n\nSCRIPT_DIR not set in /var/www/conf/openwebmail/openwebmail_path.conf !\n"; exit 0; }
 push (@INC, $SCRIPT_DIR);
 
-foreach (qw(PATH ENV BASH_ENV CDPATH IFS TERM)) { $ENV{$_}='' }	# secure ENV
+foreach (qw(ENV BASH_ENV CDPATH IFS TERM)) {delete $ENV{$_}}; $ENV{PATH}='/bin:/usr/bin'; # secure ENV
 umask(0002); # make sure the openwebmail group can write
 
 use strict;
@@ -31,8 +31,12 @@ require "modules/htmltext.pl";
 require "auth/auth.pl";
 require "quota/quota.pl";
 require "shares/ow-shared.pl";
+require "shares/iconv.pl";
 require "shares/maildb.pl";
 
+# optional module
+ow::tool::has_module('Compress/Zlib.pm');
+
 # common globals
 use vars qw(%config %config_raw);
 use vars qw($thissession);
@@ -43,7 +47,6 @@ use vars qw($quotausage $quotalimit);
 # extern vars
 use vars qw(%lang_folders %lang_text %lang_err);	# defined in lang/xy
 use vars qw($_OFFSET $_STATUS %is_internal_dbkey);	# defined in maildb.pl
-use vars qw(%is_defaultfolder @defaultfolders);		# defined in ow-shared.pl
 
 # local globals
 use vars qw($folder);
@@ -52,22 +55,20 @@ use vars qw($escapedfolder);
 
 ########## MAIN ##################################################
 openwebmail_requestbegin();
-$SIG{PIPE}=\&openwebmail_exit;	# for user stop
-$SIG{TERM}=\&openwebmail_exit;	# for user stop
-
 userenv_init();
 
 if (!$config{'enable_webmail'}) {
    openwebmailerror(__FILE__, __LINE__, "$lang_text{'webmail'} $lang_err{'access_denied'}");
 }
 
-$folder = param('folder') || 'INBOX';
+$folder = ow::tool::unescapeURL(param('folder')) || 'INBOX';
 $page = param('page') || 1;
 $sort = param('sort') || $prefs{'sort'} || 'date';
 
 $escapedfolder=ow::tool::escapeURL($folder);
 
 my $action = param('action')||'';
+writelog("debug - request folder begin, action=$action - " .__FILE__.":". __LINE__) if ($config{'debug_request'});
 if ($action eq "editfolders") {
    editfolders();
 } elsif ($action eq "refreshfolders") {
@@ -78,17 +79,18 @@ if ($action eq "editfolders") {
    reindexfolder(0);
 } elsif ($action eq "reindexfolder") {
    reindexfolder(1);
-} elsif ($action eq "addfolder") {
+} elsif ($action eq "addfolder" && $config{'enable_userfolders'}) {
    addfolder();
 } elsif ($action eq "deletefolder") {
    deletefolder();
-} elsif ($action eq "renamefolder") {
+} elsif ($action eq "renamefolder" && $config{'enable_userfolders'}) {
    renamefolder();
 } elsif ($action eq "downloadfolder") {
    downloadfolder();
 } else {
    openwebmailerror(__FILE__, __LINE__, "Action $lang_err{'has_illegal_chars'}");
 }
+writelog("debug - request folder end, action=$action - " .__FILE__.":". __LINE__) if ($config{'debug_request'});
 
 openwebmail_requestend();
 ########## END MAIN ##############################################
@@ -99,7 +101,7 @@ sub editfolders {
 
    getfolders(\@validfolders, \$inboxusage, \$folderusage);
    foreach (@validfolders) {
-      push (@userfolders, $_) if (!$is_defaultfolder{$_});
+      push (@userfolders, $_) if (!is_defaultfolder($_));
    }
 
    my $total_newmessages=0;
@@ -111,54 +113,82 @@ sub editfolders {
 
    $html =~ s/\@\@\@FOLDERNAME_MAXLEN\@\@\@/$config{'foldername_maxlen'}/g;
 
-   $temphtml = iconlink("backtofolder.gif", "$lang_text{'backto'} ".( $lang_folders{$folder}||$folder), qq|accesskey="B" href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=listmessages&amp;sessionid=$thissession&amp;sort=$sort&amp;page=$page&amp;folder=$escapedfolder"|). qq|&nbsp; \n|;
+   my $folderstr=$lang_folders{$folder}||f2u($folder);
+   $temphtml = iconlink("backtofolder.gif", "$lang_text{'backto'} $folderstr", qq|accesskey="B" href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=listmessages&amp;sessionid=$thissession&amp;sort=$sort&amp;page=$page&amp;folder=$escapedfolder"|). qq|&nbsp; \n|;
    $temphtml .= iconlink("refresh.gif", $lang_text{'refresh'}, qq|accesskey="R" href="$config{'ow_cgiurl'}/openwebmail-folder.pl?action=refreshfolders&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page"|). qq| \n|;
 
    $html =~ s/\@\@\@MENUBARLINKS\@\@\@/$temphtml/g;
 
-   $temphtml = start_form(-action=>"$config{'ow_cgiurl'}/openwebmail-folder.pl").
-               ow::tool::hiddens(action=>'addfolder',
-                                 sessionid=>$thissession,
-                                 sort=>$sort,
-                                 page=>$page,
-                                 folder=>$folder);
-   $html =~ s/\@\@\@STARTFOLDERFORM\@\@\@/$temphtml/;
-
-   $temphtml = textfield(-name=>'foldername',
-                         -default=>'',
-                         -size=> 24,
-                         -maxlength=>$config{'foldername_maxlen'},
-                         -override=>'1');
-#                         -accesskey=>'I',
-   $html =~ s/\@\@\@FOLDERNAMEFIELD\@\@\@/$temphtml/;
-
-   $temphtml = submit(-name=>$lang_text{'add'},
-                      -accesskey=>'A',
-                      -class=>"medtext");
-   $html =~ s/\@\@\@ADDBUTTON\@\@\@/$temphtml/;
-
-   $temphtml = end_form();
-   $html =~ s/\@\@\@ENDFORM\@\@\@/$temphtml/;
-
-   my $bgcolor = $style{"tablerow_dark"};
+   my $bgcolor;
    my $currfolder;
    my $form_i=0;
-   $temphtml='';
-   foreach $currfolder (@userfolders) {
-      $temphtml .= _folderline($currfolder, $form_i, $bgcolor,
-                               \$total_newmessages, \$total_allmessages, \$total_foldersize);
-      if ($bgcolor eq $style{"tablerow_dark"}) {
-         $bgcolor = $style{"tablerow_light"};
-      } else {
-         $bgcolor = $style{"tablerow_dark"};
+   if ($config{'enable_userfolders'}) {
+      templateblock_enable($html, 'USERFOLDERS');
+
+      $bgcolor = $style{"tablerow_dark"};
+      $temphtml = start_form(-action=>"$config{'ow_cgiurl'}/openwebmail-folder.pl").
+                  ow::tool::hiddens(action=>'addfolder',
+                                    sessionid=>$thissession,
+                                    sort=>$sort,
+                                    page=>$page,
+                                    folder=>$escapedfolder);
+      $html =~ s/\@\@\@STARTFOLDERFORM\@\@\@/$temphtml/;
+
+      $temphtml = textfield(-name=>'foldername',
+                            -default=>'',
+                            -size=> 24,
+                            -maxlength=>$config{'foldername_maxlen'},
+                            -override=>'1');
+#                         -accesskey=>'I',
+      $html =~ s/\@\@\@FOLDERNAMEFIELD\@\@\@/$temphtml/;
+
+      $temphtml = submit(-name=>$lang_text{'add'},
+                         -accesskey=>'A',
+                         -class=>"medtext");
+      $html =~ s/\@\@\@ADDBUTTON\@\@\@/$temphtml/;
+
+      $temphtml = end_form();
+      $html =~ s/\@\@\@ENDFORM\@\@\@/$temphtml/;
+
+      $temphtml='';
+      my ($thisfolderprefix, $lastfolderprefix);
+      my $categorizedfolders_fs = $prefs{'categorizedfolders_fs'}||'-';
+      foreach $currfolder (@userfolders) {
+         if ($prefs{'categorizedfolders'} &&
+             !is_defaultfolder($currfolder) &&
+             !is_lang_defaultfolder($currfolder)) {
+            my $folderstr=$lang_folders{$currfolder}||f2u($currfolder);
+            if ($folderstr=~/^(.+?)\Q$categorizedfolders_fs\E/) {
+               $thisfolderprefix=$1;
+               if ($thisfolderprefix ne $lastfolderprefix) {
+                  $temphtml .= qq|<tr>|.
+                               qq|<td bgcolor=$style{"columnheader"} colspan="5">&nbsp; &nbsp;<b>|.
+                               ow::htmltext::str2html($thisfolderprefix).qq|</td>|.
+                               qq|</tr>|;
+               }
+            }
+         }
+
+         $temphtml .= _folderline($currfolder, $form_i, $bgcolor,
+                                  \$total_newmessages, \$total_allmessages, \$total_foldersize);
+         if ($bgcolor eq $style{"tablerow_dark"}) {
+            $bgcolor = $style{"tablerow_light"};
+         } else {
+            $bgcolor = $style{"tablerow_dark"};
+         }
+         $form_i++;
+
+         $lastfolderprefix=$thisfolderprefix;
       }
-      $form_i++;
+      $html =~ s/\@\@\@FOLDERS\@\@\@/$temphtml/;
+
+   } else {
+      templateblock_disable($html, 'USERFOLDERS');
    }
-   $html =~ s/\@\@\@FOLDERS\@\@\@/$temphtml/;
 
    $bgcolor = $style{"tablerow_dark"};
    $temphtml='';
-   foreach $currfolder (@defaultfolders) {
+   foreach $currfolder (get_defaultfolders()) {
       $temphtml .= _folderline($currfolder, $form_i, $bgcolor,
                                \$total_newmessages, \$total_allmessages, \$total_foldersize);
       if ($bgcolor eq $style{"tablerow_dark"}) {
@@ -190,7 +220,7 @@ sub editfolders {
    $total_foldersize=lenstr($total_foldersize,0);
 
    $temphtml = qq|<tr>|.
-               qq|<td align="center" bgcolor=$bgcolor><B>$lang_text{'total'}</B></td>|.
+               qq|<td bgcolor=$bgcolor>&nbsp;<B>$lang_text{'total'}</B></td>|.
                qq|<td align="center" bgcolor=$bgcolor><B>$total_newmessages</B></td>|.
                qq|<td align="center" bgcolor=$bgcolor><B>&nbsp;$total_allmessages</B></td>|.
                qq|<td align="center" bgcolor=$bgcolor><B>&nbsp;$total_foldersize</B></td>|.
@@ -212,14 +242,14 @@ sub _folderline {
 
    if (ow::dbm::exist("$folderdb")) {
       ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} db $folderdb");
-      if ( defined($FDB{'ALLMESSAGES'}) ) {
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_readlock'} db ".f2u($folderdb));
+      if (defined $FDB{'ALLMESSAGES'}) {
          $allmessages=$FDB{'ALLMESSAGES'};
          ${$r_total_allmessages}+=$allmessages;
       } else {
          $allmessages='&nbsp;';
       }
-      if ( defined($FDB{'NEWMESSAGES'}) ) {
+      if (defined $FDB{'NEWMESSAGES'}) {
          $newmessages=$FDB{'NEWMESSAGES'};
          ${$r_total_newmessages}+=$newmessages;
       } else {
@@ -238,8 +268,8 @@ sub _folderline {
    $foldersize=lenstr($foldersize,0);
 
    my $escapedcurrfolder = ow::tool::escapeURL($currfolder);
+   my $currfolderstr=$lang_folders{$currfolder}||f2u($currfolder);
    my $url = "$config{'ow_cgiurl'}/openwebmail-folder.pl?sessionid=$thissession&amp;folder=$escapedcurrfolder&amp;action=downloadfolder";
-   my $folderstr=$lang_folders{$currfolder}||$currfolder;
 
    my $accesskeystr=$i%10+1;
    if ($accesskeystr == 10) {
@@ -248,11 +278,21 @@ sub _folderline {
       $accesskeystr=qq|accesskey="$accesskeystr"|;
    }
 
+   my ($gifstr, $folderbasename)=('', $currfolderstr);
+   if ($prefs{'categorizedfolders'} &&
+       !is_defaultfolder($currfolder) &&
+       !is_lang_defaultfolder($currfolder)) {
+      my $categorizedfolders_fs = $prefs{'categorizedfolders_fs'}||'-';
+      if ($currfolderstr=~/^.+?\Q$categorizedfolders_fs\E(.+)$/) {
+         $gifstr=qq| &nbsp; - &nbsp; |;
+         $folderbasename=$1;
+      }
+   }
    $temphtml .= qq|<tr>|.
-                qq|<td align="center" bgcolor=$bgcolor>|.
+                qq|<td bgcolor=$bgcolor> &nbsp; &nbsp; $gifstr|.
                 qq|<a href="$config{'ow_cgiurl'}/openwebmail-main.pl?action=listmessages&amp;sessionid=$thissession&amp;sort=$sort&amp;page=$page&amp;folder=$escapedcurrfolder">|.
-                ow::htmltext::str2html($folderstr).qq| </a>&nbsp;\n|.
-                iconlink("download.gif", "$lang_text{'download'} $folderstr ", qq|$accesskeystr href="$url"|).
+                ow::htmltext::str2html($folderbasename).qq| </a>&nbsp;\n|.
+                iconlink("download.gif", "$lang_text{'download'} $currfolderstr ", qq|$accesskeystr href="$url"|).
                 qq|</td>\n|.
                 qq|<td align="center" bgcolor=$bgcolor>$newmessages</td>|.
                 qq|<td align="center" bgcolor=$bgcolor>&nbsp;$allmessages</td>|.
@@ -266,12 +306,12 @@ sub _folderline {
                                   sessionid=>$thissession,
                                   sort=>$sort,
                                   page=>$page,
-                                  folder=>$folder,
-                                  foldername=>$currfolder,
-                                  foldernewname=>$currfolder)."\n";
+                                  folder=>$escapedfolder,
+                                  foldername=>$escapedcurrfolder,
+                                  foldernamestr=>$currfolderstr,
+                                  foldernewname=>'')."\n";
 
-   my $jsfolderstr=$lang_folders{$currfolder}||$currfolder;
-   $jsfolderstr=~ s/'/\\'/g;	# escaep ' with \'
+   my $jsfolderstr=$currfolderstr; $jsfolderstr=~ s/'/\\'/g;	# escaep ' with \'
    $temphtml .= submit(-name=>$lang_text{'markread'},
                        -class=>"medtext",
                        -onClick=>"return OpConfirm('folderform$i', 'markreadfolder', $lang_text{'foldermarkreadconf'}+' ( $jsfolderstr )')");
@@ -285,10 +325,10 @@ sub _folderline {
       $temphtml .= submit(-name=>$lang_text{'rename'},
                           -class=>"medtext",
                           -onClick=>"return OpConfirm('folderform$i', 'renamefolder', $lang_text{'folderrenprop'}+' ( $jsfolderstr )')");
-      $temphtml .= submit(-name=>$lang_text{'delete'},
-                          -class=>"medtext",
-                          -onClick=>"return OpConfirm('folderform$i', 'deletefolder', $lang_text{'folderdelconf'}+' ( $jsfolderstr )')");
    }
+   $temphtml .= submit(-name=>$lang_text{'delete'},
+                       -class=>"medtext",
+                       -onClick=>"return OpConfirm('folderform$i', 'deletefolder', $lang_text{'folderdelconf'}+' ( $jsfolderstr )')");
 
    $temphtml .= "</td></tr>".end_form()."\n";
 
@@ -306,8 +346,8 @@ sub refreshfolders {
    foreach my $currfolder (@validfolders) {
       my ($folderfile,$folderdb)=get_folderpath_folderdb($user, $currfolder);
 
-      ow::filelock::lock($folderfile, LOCK_EX|LOCK_NB) or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderfile!");
+      ow::filelock::lock($folderfile, LOCK_EX) or
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} ".f2u($folderfile)."!");
       if (update_folderindex($folderfile, $folderdb)<0) {
          $errcount++;
          writelog("db error - Couldn't update db $folderdb");
@@ -328,22 +368,22 @@ sub refreshfolders {
 
 ########## MARKREADFOLDER ########################################
 sub markreadfolder {
-   my $foldertomark = ow::tool::untaint(safefoldername(param('foldername'))) || '';
+   my $foldertomark = ow::tool::untaint(safefoldername(ow::tool::unescapeURL(param('foldername')))) || '';
    my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $foldertomark);
 
    my $ioerr=0;
 
-   ow::filelock::lock($folderfile, LOCK_EX|LOCK_NB) or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderfile!");
+   ow::filelock::lock($folderfile, LOCK_EX) or
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} ".f2u($folderfile)."!");
 
    if (update_folderindex($folderfile, $folderdb)<0) {
       ow::filelock::lock($folderfile, LOCK_UN);
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} db $folderdb");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} db ".f2u($folderdb));
    }
 
    my (%FDB, %offset, %status);
    ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} db $folderdb");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_readlock'} db ".f2u($folderdb));
    foreach my $messageid (keys %FDB) {
       next if ($is_internal_dbkey{$messageid});
       my @attr=string2msgattr($FDB{$messageid});
@@ -355,15 +395,15 @@ sub markreadfolder {
    ow::dbm::close(\%FDB, $folderdb);
    my @unreadmsgids=(sort { $offset{$a}<=>$offset{$b} } keys %offset);
 
-   my $tmpfile=ow::tool::untaint("/tmp/markread_tmp_$$");
-   my $tmpdb=ow::tool::untaint("/tmp/.markread_tmp_$$");
+   my $tmpfile=ow::tool::tmpname("markread.tmpfile");
+   my $tmpdb=ow::tool::tmpname("markread.tmpdb");
 
    while (!$ioerr && $#unreadmsgids>=0) {
       my @markids=();
 
       open(F, ">$tmpfile"); close(F);
       ow::filelock::lock($tmpfile, LOCK_EX) or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $tmpfile");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} $tmpfile");
 
       if (update_folderindex($tmpfile, $tmpdb)<0) {
          ow::filelock::lock($tmpfile, LOCK_UN);
@@ -375,7 +415,7 @@ sub markreadfolder {
       while (!$ioerr && $#unreadmsgids>=0) {
          my $messageid=shift(@unreadmsgids);
 
-         my $copied=operate_message_with_ids("copy", [$messageid], $folderfile, $folderdb, $tmpfile, $tmpdb);
+         my $copied=(operate_message_with_ids("copy", [$messageid], $folderfile, $folderdb, $tmpfile, $tmpdb))[0];
          if ($copied>0) {
             if (update_message_status($messageid, $status{$messageid}."R", $tmpdb, $tmpfile)==0) {
                push(@markids, $messageid);
@@ -392,10 +432,9 @@ sub markreadfolder {
               $#unreadmsgids<0 ) { 			# no more unread msg
             # copy read msg back from tmp folder
             if ($#markids>=0) {
-               $ioerr++ if (operate_message_with_ids("delete", \@markids, $folderfile, $folderdb)<0);
+               $ioerr++ if ((operate_message_with_ids("delete", \@markids, $folderfile, $folderdb))[0]<0);
                $ioerr++ if (folder_zapmessages($folderfile, $folderdb)<0);
-               $ioerr++ if (operate_message_with_ids("move", \@markids,
-   					$tmpfile, $tmpdb, $folderfile, $folderdb)<0);
+               $ioerr++ if ((operate_message_with_ids("move", \@markids, $tmpfile, $tmpdb, $folderfile, $folderdb))[0]<0);
             }
             last;	# renew tmp folder and @markids
          }
@@ -418,23 +457,27 @@ sub markreadfolder {
 ########## REINDEXFOLDER #########################################
 sub reindexfolder {
    my $recreate=$_[0];
-   my $foldertoindex = ow::tool::untaint(safefoldername(param('foldername'))) || '';
+   my $foldertoindex = ow::tool::untaint(safefoldername(ow::tool::unescapeURL(param('foldername')))) || '';
    my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $foldertoindex);
 
+   ow::filelock::lock($folderfile, LOCK_SH|LOCK_NB) or
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_readlock'} ".f2u($folderfile));
+
    if ($recreate) {
       ow::dbm::unlink($folderdb);
    }
    if (ow::dbm::exist($folderdb) ) {
       my %FDB;
-      ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} db $folderdb");
-
-      $FDB{'METAINFO'}={'RENEW'};
+      if (!ow::dbm::open(\%FDB, $folderdb, LOCK_SH)) {
+         ow::filelock::lock($folderfile, LOCK_UN);
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_readlock'} db ".f2u($folderdb));
+      }
+      @FDB{'METAINFO', 'LSTMTIME'}=('RENEW', -1);
       ow::dbm::close(\%FDB, $folderdb);
    }
    if (update_folderindex($folderfile, $folderdb)<0) {
       ow::filelock::lock($folderfile, LOCK_UN);
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} db $folderdb");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} db ".f2u($folderdb));
    }
 
    if ($recreate) {
@@ -447,6 +490,8 @@ sub reindexfolder {
       writehistory("chkindex folder - $foldertoindex");
    }
 
+   ow::filelock::lock($folderfile, LOCK_UN);
+
    editfolders();
 }
 ########## END REINDEXFOLDER #####################################
@@ -460,28 +505,36 @@ sub addfolder {
       }
    }
 
-   my $foldertoadd = ow::tool::untaint(safefoldername(param('foldername'))) || '';
+   my $foldertoadd = ow::tool::untaint(ow::tool::unescapeURL(param('foldername'))) || ''; # from js field
+   $foldertoadd = u2f($foldertoadd);
+   is_safefoldername($foldertoadd) or
+      openwebmailerror(__FILE__, __LINE__, "$foldertoadd $lang_err{'has_illegal_chars'}");
+   $foldertoadd = safefoldername($foldertoadd);
+   return editfolders() if ($foldertoadd eq '');
+
    if (length($foldertoadd) > $config{'foldername_maxlen'}) {
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'foldername_long'}");
+      my $msg=$lang_err{'foldername_long'};
+      $msg =~ s/\@\@\@FOLDERNAME_MAXLEN\@\@\@/$config{'foldername_maxlen'}/;
+      openwebmailerror(__FILE__, __LINE__, $msg);
    }
-   if ( is_defaultfolder($foldertoadd) ||
-        $foldertoadd eq "$user" || $foldertoadd eq "" ) {
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_folder'}");
+   if ( is_defaultfolder($foldertoadd) || is_lang_defaultfolder($foldertoadd) ||
+        $foldertoadd eq "$user") {
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_folder'} (".f2u($foldertoadd).")");
    }
 
    my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $foldertoadd);
    if ( -f $folderfile ) {
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'folder_with_name'} $foldertoadd $lang_err{'already_exists'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'folder_with_name'} ".f2u($foldertoadd)." $lang_err{'already_exists'}");
    }
 
    open (FOLDERTOADD, ">$folderfile") or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_folder'} $foldertoadd! ($!)");
-   close (FOLDERTOADD) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} $foldertoadd! ($!)");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_folder'} ".f2u($foldertoadd)."! ($!)");
+   close (FOLDERTOADD) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} ".f2u($foldertoadd)."! ($!)");
 
    # create empty index dbm with mode 0600
    my %FDB;
    ow::dbm::open(\%FDB, $folderdb, LOCK_EX) or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdb");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} ".f2u($folderdb));
    ow::dbm::close(\%FDB, $folderdb);
 
    writelog("create folder - $foldertoadd");
@@ -490,10 +543,9 @@ sub addfolder {
    editfolders();
 }
 
-sub is_defaultfolder {
-   return 1 if ($is_defaultfolder{$_[0]});
-   foreach (keys %is_defaultfolder) {
-      return 1 if ($_[0] eq $lang_folders{$_})
+sub is_lang_defaultfolder {
+   foreach (keys %lang_folders) {	# defaultfolder locallized name check
+      return 1 if ($_[0] eq $lang_folders{$_} && is_defaultfolder($_));
    }
    return 0;
 }
@@ -501,13 +553,7 @@ sub is_defaultfolder {
 
 ########## DELETEFOLDER ##########################################
 sub deletefolder {
-   my $foldertodel = safefoldername(param('foldername')) || '';
-
-   # if is INBOX, return to editfolder immediately
-   if ($foldertodel eq 'INBOX') {
-      editfolders();
-      return;
-   }
+   my $foldertodel = safefoldername(ow::tool::unescapeURL(param('foldername'))) || '';
 
    my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $foldertodel);
    if ( -f $folderfile) {
@@ -530,18 +576,25 @@ sub deletefolder {
 
 ########## RENAMEFOLDER ##########################################
 sub renamefolder {
-   my $oldname = ow::tool::untaint(safefoldername(param('foldername'))) || '';
+   my $oldname = ow::tool::untaint(safefoldername(ow::tool::unescapeURL(param('foldername')))) || '';
    if ($oldname eq 'INBOX') {
-      editfolders();
-      return;
+      return editfolders();
    }
 
-   my $newname = ow::tool::untaint(safefoldername(param('foldernewname')))||'';
+   my $newnamestr = ow::tool::untaint(param('foldernewname'))||'';	# from js field
+   my $newname = u2f($newnamestr);
+   is_safefoldername($newname) or
+      openwebmailerror(__FILE__, __LINE__, "$newname $lang_err{'has_illegal_chars'}");
+   $newname = safefoldername($newname);
+   return editfolders() if ($newname eq '');
+
    if (length($newname) > $config{'foldername_maxlen'}) {
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'foldername_long'}");
+      my $msg=$lang_err{'foldername_long'};
+      $msg =~ s/\@\@\@FOLDERNAME_MAXLEN\@\@\@/$config{'foldername_maxlen'}/;
+      openwebmailerror(__FILE__, __LINE__, $msg);
    }
-   if ( is_defaultfolder($newname) ||
-        $newname eq "$user" || $newname eq "" ) {
+   if ( is_defaultfolder($newname) || is_lang_defaultfolder($newname) ||
+        $newname eq "$user") {
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_folder'}");
    }
 
@@ -549,7 +602,7 @@ sub renamefolder {
    my ($newfolderfile, $newdb)=get_folderpath_folderdb($user, $newname);
 
    if ( -f $newfolderfile ) {
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'folder_with_name'} $newname $lang_err{'already_exists'}");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'folder_with_name'} $newnamestr $lang_err{'already_exists'}");
    }
 
    if ( -f $oldfolderfile ) {
@@ -573,7 +626,7 @@ sub downloadfolder {
    my $buff;
 
    if ( ($cmd=ow::tool::findbin("zip")) ne "" ) {
-      $cmd.=" -rq - $folderfile |";
+      $cmd.=" -jrq - $folderfile |";
       $contenttype='application/x-zip-compressed';
       $filename="$folder.zip";
 
@@ -590,8 +643,8 @@ sub downloadfolder {
 
    $filename=~s/\s+/_/g;
 
-   ow::filelock::lock($folderfile, LOCK_EX|LOCK_NB) or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderfile");
+   ow::filelock::lock($folderfile, LOCK_EX) or
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} ".f2u($folderfile));
 
    # disposition:attachment default to save
    print qq|Connection: close\n|,
