@@ -28,6 +28,8 @@ if ($dictionary_letters{english} eq '') {
    french    => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',
    magyar    => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz~',
    polski    => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz󶼿ʣӦ',
+   slovensko => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',
+   spanish   => 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz',
    ukrainian => 'ȧƦ֤\'',
    );
 }
@@ -56,14 +58,14 @@ if (!$memdic{a}) {
 }
 
 use vars qw($SCRIPT_DIR);
-if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1 }
+if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { local $1; $SCRIPT_DIR=$1 }
 if ($SCRIPT_DIR eq '' && open(F, '/var/www/conf/openwebmail/openwebmail_path.conf')) {
-   $_=<F>; close(F); if ( $_=~/^(\S*)/) { $SCRIPT_DIR=$1 }
+   $_=<F>; close(F); if ( $_=~/^(\S*)/) { local $1; $SCRIPT_DIR=$1 }
 }
 if ($SCRIPT_DIR eq '') { print "Content-type: text/html\n\nSCRIPT_DIR not set in /var/www/conf/openwebmail/openwebmail_path.conf !\n"; exit 0; }
 push (@INC, $SCRIPT_DIR);
 
-foreach (qw(PATH ENV BASH_ENV CDPATH IFS TERM)) { $ENV{$_}='' }	# secure ENV
+foreach (qw(ENV BASH_ENV CDPATH IFS TERM)) {delete $ENV{$_}}; $ENV{PATH}='/bin:/usr/bin'; # secure ENV
 umask(0002); # make sure the openwebmail group can write
 
 use strict;
@@ -83,6 +85,9 @@ require "auth/auth.pl";
 require "quota/quota.pl";
 require "shares/ow-shared.pl";
 
+# optional module
+ow::tool::has_module('Compress/Zlib.pm');
+
 # common globals
 use vars qw(%config %config_raw);
 use vars qw($thissession);
@@ -94,10 +99,6 @@ use vars qw(%lang_text %lang_err);	# defined in lang/xy
 
 ########## MAIN ##################################################
 openwebmail_requestbegin();
-$SIG{PIPE}=\&openwebmail_exit;	# for user stop
-$SIG{TERM}=\&openwebmail_exit;	# for user stop
-$SIG{CHLD}='IGNORE';		# avoid zombies
-
 userenv_init();
 
 if (!$config{'enable_webmail'} || !$config{'enable_spellcheck'}) {
@@ -111,35 +112,37 @@ my $form = param('form')||'';
 my $field = param('field')||'';
 my $dictionary = param('dictionary') || $prefs{'dictionary'} || 'english';
 my $dicletters=$dictionary_letters{'english'};
-$dicletters=$dictionary_letters{$dictionary} if (defined($dictionary_letters{$dictionary}));
+$dicletters=$dictionary_letters{$dictionary} if (defined $dictionary_letters{$dictionary});
 
 my $spellbin=(split(/\s+/, $config{'spellcheck'}))[0];
 if (! -x $spellbin) {
    openwebmailerror(__FILE__, __LINE__, "Spellcheck is not available.<br>( $spellbin not found )");
 }
 
-if (defined(param('string'))) {
+writelog("debug - request spell begin - " .__FILE__.":". __LINE__) if ($config{'debug_request'});
+if (defined param('string')) {
    my ($wordcount, $wordframe, @words)=text2words($htmlmode, param('string')||'', $dicletters);
    my ($wordshtml, $error)=spellcheck_words2html($htmlmode, $wordcount, \$wordframe, \@words, $dictionary);
    docheckform($htmlmode, $form, $field, $dictionary, $wordshtml, $error, $wordcount, $wordframe);
 
-} elsif (defined(param('checkagainbutton'))) {
+} elsif (defined param('checkagainbutton')) {
    my ($wordcount, $wordframe, @words)=cgiparam2words();
    my ($wordshtml, $error)=spellcheck_words2html($htmlmode, $wordcount, \$wordframe, \@words, $dictionary);
    docheckform($htmlmode, $form, $field, $dictionary, $wordshtml, $error, $wordcount, $wordframe);
 
-} elsif (defined(param('finishcheckingbutton'))) {
+} elsif (defined param('finishcheckingbutton')) {
    my ($wordcount, $wordframe, @words)=cgiparam2words();
    spellcheck_words2html($htmlmode, $wordcount, \$wordframe, \@words, $dictionary);	# for updating pdict
    my $finalstring=words2text(\$wordframe, \@words, $dicletters);
    finalform($form, $field, $finalstring);
 
-} elsif (defined(param('editpdictbutton'))) {
+} elsif (defined param('editpdictbutton')) {
    editpdict(param('dictword2delete')||'', $dictionary);
 
 } else {
    httpprint([], [htmlheader(), "What the heck? Invalid input for Spellcheck!", htmlfooter(1)]);
 }
+writelog("debug - request spell end - " .__FILE__.":". __LINE__) if ($config{'debug_request'});
 
 openwebmail_requestend();
 ########## END MAIN ##############################################
@@ -167,10 +170,10 @@ sub docheckform {
                                  field=>$fieldname,
                                  dictionary=>$dictionary,
                                  wordcount=>$wordcount,
-                                 wordframe=>ow::tool::escapeURL($wordframe));
+                                 wordframe=>ow::htmltext::str2html($wordframe));
    $html =~ s/\@\@\@STARTSPELLCHECKFORM\@\@\@/$temphtml/;
 
-   if ( defined(param('checkagainbutton')) ) {
+   if (defined param('checkagainbutton')) {
       $temphtml = button(-name=>'backbutton',
                          -value=>$lang_err{'back'},
                          -onclick=>'window.history.back();',
@@ -179,7 +182,7 @@ sub docheckform {
       $temphtml = "";
    }
    if ($error>0) {
-      $temphtml .= "&nbsp;&nbsp;" if (defined(param('checkagainbutton')));
+      $temphtml .= "&nbsp;&nbsp;" if (defined param('checkagainbutton'));
       $temphtml .= submit(-name=>'checkagainbutton',
                           -value=>$lang_text{'checkagain'},
                           -override=>'1');
@@ -339,7 +342,7 @@ sub text2words {
               "isn't wasn't aren't weren't ";
 
    # put url to ignore
-   foreach my $word ($text=~m![A-Za-z]+tp://[A-Za-z\d\.]+!ig) {
+   foreach my $word ($text=~m![A-Za-z]+tp://[^\s]+!ig) {
       $ignore.=" $word";
    }
    # put email to ignore
@@ -380,7 +383,7 @@ sub _word2label {
    my ($word, $wordignore, $r_wordcount, $r_words, $r_wordnums)=@_;
    return($word) if ($memdic{lc($word)} || $wordignore=~/\Q$word\E/i ||
                      $word =~/^WORD/ || $word =~/^TAG/);
-   return('%%WORD'.${$r_wordnums}{$word}.'%%') if (defined(${$r_wordnums}{$word}));
+   return('%%WORD'.${$r_wordnums}{$word}.'%%') if (defined ${$r_wordnums}{$word});
 
    my $label='%%WORD'.${$r_wordcount}.'%%';
    ${$r_words}[${$r_wordcount}]=$word;
@@ -398,9 +401,9 @@ sub cgiparam2words {
 
    my $newwordcount=0;
    for (my $i=0; $i<$wordcount; $i++) {
-      if (defined(param($i))) {
+      if (defined param($i)) {
          my $word=param($i);
-         if (!defined($wordnums{$word})) {
+         if (!defined $wordnums{$word}) {
             $words[$i]=$word;
             $wordnums{$word}=$i;
             $newwordcount=$i+1;
@@ -456,12 +459,7 @@ sub spellcheck_words2html {
    }
 
    # conversion make text for happy html display
-   $html=~s/&/&amp;/g;
-   $html=~s/</&lt;/g;
-   $html=~s/>/&gt;/g;
-   $html=~s/\n/<BR>/g;
-   $html=~s/"/&quot;/g;
-   $html=~s/  /&nbsp;&nbsp;/g;
+   $html=ow::htmltext::text2html_nolink($html);
 
    # find all words leading with ~!~, remove ~!~ and add them to pdict
    my %pdicword=();
@@ -477,11 +475,14 @@ sub spellcheck_words2html {
       # add words to person dict
       # the 2nd \n guarentees we have output in piperead
       pipewrite($spellcmd."\#\n\n");
-      ($stdout, $stderr)=piperead();
-      if ($stderr=~/[^\s]/) {
-         pipeclose();
-         openwebmailerror(__FILE__, __LINE__, "Spellcheck error: $stderr");
-      }
+      ($stdout, $stderr)=piperead(2);
+
+      # it seems adding words to pdict doesn't generate output on aspell 0.50,
+      # so we comment out the result check here
+      # if ($stderr=~/[^\s]/) {
+      #    pipeclose();
+      #    openwebmailerror(__FILE__, __LINE__, "Spellcheck error: $stderr");
+      # }
    }
 
    my %dupwordhtml=();
@@ -490,10 +491,10 @@ sub spellcheck_words2html {
       my $word=${$r_words}[$i];
       my $wordhtml='';
 
-      if (defined($dupwordhtml{$word})) {	# different symbo with duplicate word
+      if (defined $dupwordhtml{$word}) {	# different symbo with duplicate word
          $wordhtml=$dupwordhtml{$word};
 
-      } elsif (defined($pdicword{$word})) {	# words already put into pdic
+      } elsif (defined $pdicword{$word}) {	# words already put into pdic
          $wordhtml=$dupwordhtml{$word}=$word;
 
       } elsif ( $word=~/^~~/ ) {	# check if manualfix
@@ -621,10 +622,11 @@ use vars qw(*spellIN *spellOUT *spellERR);
 use vars qw($mypid $pipepid $piperun $pipeexit $pipesig);
 $mypid=$$;
 sub pipeopen {
+   local $1; # fix perl $1 taintness propagation bug
    my @cmd=@_; foreach (@cmd) { (/^(.*)$/) && ($_=$1) };	# untaint all argument
    local $|=1;				# flush CGI related output in parent
    ($piperun, $pipeexit, $pipesig)=(1,0,0);
-   $SIG{CHLD}=sub { wait; $pipeexit=$?>>8; $pipesig=$?&255; $piperun=0; };
+   local $SIG{CHLD}=sub { wait; $pipeexit=$?>>8; $pipesig=$?&255; $piperun=0; }; # to get child status
    eval { $pipepid = open3(\*spellIN, \*spellOUT, \*spellERR, @cmd); };
    if ($@) {			# open3 return err only in child
       if ($$!=$mypid){ 		# child
@@ -664,7 +666,7 @@ sub piperead {
          if ($stdout=~/\n/||$stderr=~/\n/) {	# data line already in
             last;
          } elsif ($stdout eq "" && $stderr eq "") {	# 1st read timeout
-            $stderr="piperead nothing"; last;	
+            $stderr="piperead nothing"; last;
          } # else continue to read until line
 
       } else {	# n<0, read err => child dead?
