@@ -6,9 +6,9 @@
 #            Ebola.AT.turtle.ee.ncku.edu.tw
 #
 use vars qw($SCRIPT_DIR);
-if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1 }
+if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { local $1; $SCRIPT_DIR=$1 }
 if ($SCRIPT_DIR eq '' && open(F, '/var/www/conf/openwebmail/openwebmail_path.conf')) {
-   $_=<F>; close(F); if ( $_=~/^(\S*)/) { $SCRIPT_DIR=$1 }
+   $_=<F>; close(F); if ( $_=~/^(\S*)/) { local $1; $SCRIPT_DIR=$1 }
 }
 if ($SCRIPT_DIR eq '') {
    print qq|\nOpen WebMail is unable to locate itself on this system,\n|.
@@ -16,15 +16,14 @@ if ($SCRIPT_DIR eq '') {
          qq|the first line of file /var/www/conf/openwebmail/openwebmail_path.conf\n\n|.
          qq|For example, if the script is\n\n|.
          qq|/var/www/cgi-bin/openwebmail/openwebmail-tool.pl,\n\n|.
-         qq|then the content of /var/www/conf/openwebmail/openwebmail_path.conf should be:\n\n|.
+         qq|then the content of /etc/openwebmail_path.conf should be:\n\n|.
          qq|/var/www/cgi-bin/openwebmail/\n\n|;
    exit 0;
 }
 push (@INC, $SCRIPT_DIR);
 
-foreach (qw(PATH ENV BASH_ENV CDPATH IFS TERM)) { $ENV{$_}='' }	# secure ENV
+foreach (qw(ENV BASH_ENV CDPATH IFS TERM)) {delete $ENV{$_}}; $ENV{PATH}='/bin:/usr/bin'; # secure ENV
 umask(0002); # make sure the openwebmail group can write
-
 use strict;
 use Fcntl qw(:DEFAULT :flock);
 use Net::SMTP;
@@ -38,24 +37,34 @@ require "modules/datetime.pl";
 require "modules/lang.pl";
 require "modules/mime.pl";
 require "modules/mailparse.pl";
-require "modules/pop3.pl";
+require "modules/spamcheck.pl";
+require "modules/viruscheck.pl";
 require "auth/auth.pl";
 require "quota/quota.pl";
 require "shares/ow-shared.pl";
+require "shares/iconv.pl";
 require "shares/maildb.pl";
+require "shares/lockget.pl";
 require "shares/cut.pl";
-require "shares/mailfilter.pl";
+require "shares/getmsgids.pl";
+require "shares/fetchmail.pl";
 require "shares/pop3book.pl";
+require "shares/adrbook.pl";
 require "shares/calbook.pl";
+require "shares/filterbook.pl";
+require "shares/mailfilter.pl";
 require "shares/lunar.pl";
-require "shares/iconv-chinese.pl";
 require "shares/upgrade.pl";
 
+# optional module
+ow::tool::has_module('IO/Socket/SSL.pm');
+
 # common globals
 use vars qw(%config %config_raw);
 use vars qw($default_logindomain $loginname $logindomain $loginuser);
 use vars qw($domain $user $userrealname $uuid $ugid $homedir);
 use vars qw(%prefs);
+use vars qw(%lang_text);
 
 # extern vars
 use vars qw(%is_config_option);		# from ow-shared.pl
@@ -65,11 +74,9 @@ use vars qw(%is_internal_dbkey);	# from maildb.pl
 # local globals
 use vars qw($POP3_TIMEOUT %opt);
 
+
 ########## main ##################################################
 openwebmail_requestbegin();
-$SIG{PIPE}=\&openwebmail_exit;	# for user stop
-$SIG{TERM}=\&openwebmail_exit;	# for user stop
-$SIG{CHLD}='IGNORE';		# prevent zombie
 
 $POP3_TIMEOUT=20;
 
@@ -94,6 +101,10 @@ if ($ARGV[0] eq "--") {		# called by inetd
          $opt{'init'}=1;
       } elsif ($ARGV[$i] eq "--test") {
          $opt{'test'}=1;
+      } elsif ($ARGV[$i] eq "--langconv") {
+         $opt{'langconv'}=1;
+         $i++; $opt{'srclang'}=$ARGV[$i];
+         $i++; $opt{'dstlang'}=$ARGV[$i];
 
       } elsif ($ARGV[$i] eq "--yes" || $ARGV[$i] eq "-y") {
          $opt{'yes'}=1;
@@ -121,7 +132,7 @@ if ($ARGV[0] eq "--") {		# called by inetd
          $opt{'thumbnail'}=1;
 
       } elsif ($ARGV[$i] eq "--index" || $ARGV[$i] eq "-i") {
-         $opt{'iv'}='ALL'; $opt{'null'}=0; $euid_to_use=$<;
+         $opt{'iv'}='ALL'; $opt{'null'}=0;
       } elsif ($ARGV[$i] eq "-id") {
          $i++; $opt{'id'}=$ARGV[$i]; $opt{'null'}=0;
       } elsif ($ARGV[$i] eq "-iv") {
@@ -144,8 +155,12 @@ if ($ARGV[0] eq "--") {		# called by inetd
          $opt{'pop3'}=1; $opt{'null'}=0;
       } elsif ($ARGV[$i] eq "--size" || $ARGV[$i] eq "-s") {
          $opt{'size'}=1; $opt{'null'}=0;
+      } elsif ($ARGV[$i] eq "--unlock" || $ARGV[$i] eq "-u") {
+         $opt{'unlock'}=1; $opt{'null'}=0;
       } elsif ($ARGV[$i] eq "--zaptrash" || $ARGV[$i] eq "-z") {
          $opt{'zap'}=1; $opt{'null'}=0;
+      } elsif ($ARGV[$i] eq "--convert_addressbooks" || $ARGV[$i] eq "-c") {
+         $opt{'convert_addressbooks'}=1; $opt{'null'}=0;
 
       } else {
          push(@list, $ARGV[$i]);
@@ -155,24 +170,61 @@ if ($ARGV[0] eq "--") {		# called by inetd
 
 $>=$euid_to_use;
 
+load_owconf(\%config_raw, "/var/www/conf/openwebmail/defaults/openwebmail.conf");
+if ( -f "/var/www/conf/openwebmail/openwebmail.conf") {
+   read_owconf(\%config, \%config_raw, "/var/www/conf/openwebmail/openwebmail.conf");
+   print "D readconf /var/www/conf/openwebmail/openwebmail.conf\n" if ($opt{'debug'});
+}
+
+$logindomain=$default_logindomain||ow::tool::hostname();
+$logindomain=lc(safedomainname($logindomain));
+if (defined $config{'domainname_equiv'}{'map'}{$logindomain}) {
+   print "D domain equivalence found: $logindomain -> $config{'domainname_equiv'}{'map'}{$logindomain}\n" if ($opt{'debug'});
+   $logindomain=$config{'domainname_equiv'}{'map'}{$logindomain};
+}
+if ( -f "$config{'ow_sitesconfdir'}/$logindomain") {
+   read_owconf(\%config, \%config_raw, "$config{'ow_sitesconfdir'}/$logindomain");
+   print "D readconf $config{'ow_sitesconfdir'}/$logindomain\n" if ($opt{'debug'});
+}
+
+%prefs = readprefs();
+loadlang($prefs{'language'});	# for converted filename $lang_text{abook_converted}
+
+writelog("debug - request tool begin, argv=".join(' ',@ARGV)." - " .__FILE__.":". __LINE__) if ($config{'debug_request'});
 my $retval=0;
 if ($opt{'init'}) {
    $retval=init();
 } elsif ($opt{'test'}) {
    $retval=do_test();
+} elsif ($opt{'langconv'}) {
+   $retval=langconv($opt{'srclang'}, $opt{'dstlang'});
 } elsif ($opt{'thumbnail'}) {
    $retval=makethumbnail(\@list);
 } else {
+   if ($opt{'convert_addressbooks'} && $>==0) {	# only allow root to convert globalbook
+      print "converting GLOBAL addressbook..." if (!$opt{'quiet'});
+      $retval=convert_addressbook('global', $prefs{'charset'});
+      if ($retval<0) {
+         print "error:$@. EXITING\n";
+         openwebmail_exit($retval);
+      }
+      print "done.\n" if (!$opt{'quiet'});
+   }
+
    if ($opt{'allusers'}) {
       $retval=allusers(\@list);
       openwebmail_exit($retval) if ($retval<0);
    }
+
    if ($#list>=0 && !$opt{'null'}) {
       $retval=usertool($euid_to_use, \@list);
+   } elsif ($opt{'convert_addressbooks'}) {
+      # don't show help after converting just GLOBAL book
    } else {
       $retval=showhelp();
    }
 }
+writelog("debug - request tool end, argv=".join(' ',@ARGV)." - " .__FILE__.":". __LINE__) if ($config{'debug_request'});
 
 openwebmail_exit($retval);
 
@@ -181,6 +233,7 @@ sub showhelp {
    print "
 Syntax: openwebmail-tool.pl --init [options]
         openwebmail-tool.pl --test
+        openwebmail-tool.pl --langconv srclang dstlang
         openwebmail-tool.pl -t [options] [image1 image2 ...]
         openwebmail-tool.pl [options] [user1 user2 ...]
 
@@ -211,7 +264,9 @@ mail/calendar options:
  -n, --notify \t check and send calendar notification email
  -p, --pop3   \t fetch pop3 mail for user
  -s, --size   \t check user quota and cut mails/files if over quotalimit
+ -u, --unlock  \t remove file locks related to user
  -z, --zaptrash\t remove stale messages from trash folder
+ -c, --convert_addressbooks\t convert addressbookglobal (and all users with -a) addressbooks to vcard format
 
 ps: <folder> can be INBOX, ALL or folder filename
 
@@ -225,7 +280,7 @@ sub init {
 
    my $err=do_test(1);
    if ($err<0) {
-      print qq|And execute '$SCRIPT_DIR/openwebmail-tool.pl --init' again!\n\n|.
+      print qq|And execute '/var/www/conf/openwebmail/openwebmail-tool.pl --init' again!\n\n|.
             qq|ps: If you are running openwebmail in persistent mode,\n|.
             qq|    don't forget to 'touch openwebmail*.pl', so speedycgi\n|.
             qq|    will reload all scripts, modules and conf files in --init.\n\n|;
@@ -234,7 +289,7 @@ sub init {
 
    foreach my $table ('b2g', 'g2b', 'lunar') {
       if ( $config{$table.'_map'} ) {
-         my $tabledb="$config{'ow_etcdir'}/$table";
+         my $tabledb="$config{'ow_mapsdir'}/$table";
          my $err=0;
          if (ow::dbm::exist($tabledb)) {
             my %T;
@@ -246,7 +301,7 @@ sub init {
          }
          if ( !ow::dbm::exist($tabledb)) {
             die "$config{$table.'_map'} not found" if (!-f $config{$table.'_map'});
-            print "creating db $config{'ow_etcdir'}/$table ...";
+            print "creating db $config{'ow_mapsdir'}/$table ...";
 
             $err=-2 if ($table eq 'b2g' and mkdb_b2g()<0);
             $err=-3 if ($table eq 'g2b' and mkdb_g2b()<0);
@@ -304,20 +359,13 @@ sub init {
 sub do_test {
    my ($in_init)=@_;
    my $err=0;
-   print "\n";
+   print "\n" if (!$in_init);
 
-   load_owconf(\%config_raw, "/var/www/conf/openwebmail/openwebmail.conf.default");
-   if ( -f "/var/www/conf/openwebmail/openwebmail.conf") {
-      read_owconf(\%config, \%config_raw, "/var/www/conf/openwebmail/openwebmail.conf");
-      print "D readconf /var/www/conf/openwebmail/openwebmail.conf\n" if ($opt{'debug'});
-   }
-
-   $logindomain=$default_logindomain||ow::tool::hostname();
-   $logindomain=lc(safedomainname($logindomain));
-   $logindomain=$config{'domainname_equiv'}{'map'}{$logindomain} if (defined($config{'domainname_equiv'}{'map'}{$logindomain}));
-   if ( -f "$config{'ow_sitesconfdir'}/$logindomain") {
-      read_owconf(\%config, \%config_raw, "$config{'ow_sitesconfdir'}/$logindomain");
-      print "D readconf $config{'ow_sitesconfdir'}/$logindomain\n" if ($opt{'debug'});
+   if ($MIME::Base64::VERSION < 3.00) {
+      $err--;
+      print "Base64.pm\t\t$INC{'MIME/Base64.pm'}\n\n";
+      print "Your MIME::Base64 module is too old ($MIME::Base64::VERSION),\n".
+            "please update to 3.00 or later.\n\n\n";
    }
 
 # disable this warning to make user happy...
@@ -339,7 +387,7 @@ sub do_test {
 
 sub check_tell_bug {
    my $offset;
-   my $testfile=ow::tool::untaint("/tmp/testfile.$$");
+   my $testfile=ow::tool::tmpname('test.tmpfile');
 
    open(F, ">$testfile"); print F "test"; close(F);
    open(F, ">>$testfile"); $offset=tell(F); close(F);
@@ -357,7 +405,7 @@ sub check_tell_bug {
 }
 
 sub print_dbm_module {
-   print "You perl uses the following packages for dbm:\n\n";
+   print "Your perl uses the following packages for dbm:\n\n";
    my @pm;
    foreach (keys %INC) { push (@pm, $_) if (/DB.*File/); }
    foreach (sort @pm) { print "$_\t\t$INC{$_}\n"; }
@@ -443,6 +491,108 @@ sub check_savedsuid_support {
    return 0;
 }
 
+########## langconv routines #####################################
+sub langconv {
+   my ($srclang, $dstlang)=@_;
+
+   print "langconv $srclang -> $dstlang\n";
+
+   if (!defined $ow::lang::languagecharsets{$srclang}) {
+      die "src lang $srclang is not defined in lang.pl";
+   }
+   if (!defined $ow::lang::languagecharsets{$dstlang}) {
+      die "dst lang $dstlang is not defined in lang.pl";
+   }
+   my $srccharset=$ow::lang::languagecharsets{$srclang};
+   my $dstcharset=$ow::lang::languagecharsets{$dstlang};
+   if (!is_convertible($srccharset, $dstcharset)) {
+      die "lang $srclang -> $dstlang is not convertible";
+   }
+
+   langconv_file("$config{'ow_langdir'}/$srclang", $srclang,
+                 "$config{'ow_langdir'}/$dstlang", $dstlang, 1);
+
+   langconv_dir("$config{'ow_templatesdir'}/$srclang", $srclang,
+                "$config{'ow_templatesdir'}/$dstlang", $dstlang);
+
+   langconv_dir("$config{'ow_htmldir'}/javascript/htmlarea.openwebmail/popups/$srclang", $srclang,
+                "$config{'ow_htmldir'}/javascript/htmlarea.openwebmail/popups/$dstlang", $dstlang);
+
+   return 0;
+}
+
+sub langconv_dir {
+   my ($srcdir, $srclang, $dstdir, $dstlang)=@_;
+
+   print "langconv dir $srcdir -> $dstdir\n";
+
+   die "srcdir $srcdir doesn't exist" if (!-d $srcdir);
+   if (!-d $dstdir) {
+      $dstdir=ow::tool::untaint($dstdir);
+      mkdir($dstdir, 0755) || die "create $dstdir error ($!)";
+      chmod(0755, $dstdir);
+   }
+
+   my @files;
+
+   opendir(D, $srcdir) || die "can't open directory: $srcdir";
+   while (my $f=readdir(D)) {
+      next if ($f =~ /^\..*/);
+      next if ($f =~ /.*README.*/);
+      push(@files, $f);
+   }
+   closedir(D);
+
+   foreach my $f (@files) {
+      langconv_file("$srcdir/$f", $srclang,
+                    "$dstdir/$f", $dstlang, 0);
+   }
+}
+
+sub langconv_file {
+   my ($srcfile, $srclang, $dstfile, $dstlang, $check_pkgname)=@_;
+
+   print "langconv file $srcfile -> $dstfile\n";
+
+   my @lines;
+   open(F, $srcfile) || die "$srcfile open error ($!)";
+   @lines=<F>;
+   close(F);
+
+   my $srccharset=$ow::lang::languagecharsets{$srclang};
+   my $srcpkg=$srclang; $srcpkg=~s/[\.\-]/_/g;
+
+   my $dstcharset=$ow::lang::languagecharsets{$dstlang};
+   my $dstpkg=$dstlang; $dstpkg=~s/[\.\-]/_/g;
+
+   if ($check_pkgname) {
+      if ($lines[0] ne "package ow::$srcpkg;\n") {
+         die "$srcfile 1st line is not 'package ow::$srcpkg;'";
+      }
+      $lines[0]="package ow::$dstpkg;\n";
+   }
+
+   # change charset name in html file
+   foreach (@lines) {
+      s!content="text/html; charset=$srccharset"!content="text/html; charset=$dstcharset"!ig;
+      s!charset: $srccharset!charset: $dstcharset!ig;
+   }
+
+   my $content=join('', @lines);
+   ($content)=iconv($srccharset, $dstcharset, $content);
+
+   $dstfile=ow::tool::untaint($dstfile);
+   open(F, ">$dstfile") || die "$dstfile open error ($!)";
+   print F $content;
+   close(F);
+
+   my ($fmode, $fuid, $fgid)=(stat($srcfile))[2,4,5];
+   chmod(ow::tool::untaint($fmode), $dstfile);
+   chown(ow::tool::untaint($fuid), ow::tool::untaint($fgid), $dstfile);
+
+   return 0;
+}
+
 ########## make_thumbnail ########################################
 sub makethumbnail {
    my $r_files=$_[0];
@@ -520,24 +670,14 @@ sub allusers {
    my $loaded_domain=0;
    my %userhash=();
 
-   load_owconf(\%config_raw, "/var/www/conf/openwebmail/openwebmail.conf.default");
-   if ( -f "/var/www/conf/openwebmail/openwebmail.conf") {
-      read_owconf(\%config, \%config_raw, "/var/www/conf/openwebmail/openwebmail.conf");
-      print "D readconf /var/www/conf/openwebmail/openwebmail.conf\n" if ($opt{'debug'});
-   }
-
    # trap this once now.  Let usertool() test it at the domain level later
    if ( $>!=0 &&	# setuid is required if spool is located in system dir
+        !$config{'use_homedirspools'} &&
        ($config{'mailspooldir'} eq "/var/mail" ||
         $config{'mailspooldir'} eq "/var/spool/mail")) {
       print "This operation is only available to root\n"; openwebmail_exit(0);
    }
 
-   my $logindomain=$default_logindomain||ow::tool::hostname();
-   $logindomain=lc(safedomainname($logindomain));
-   $logindomain=$config{'domainname_equiv'}{'map'}{$logindomain} if (defined($config{'domainname_equiv'}{'map'}{$logindomain}));
-   print "D found default domain $logindomain\n" if ($opt{'debug'});
-
    # if there's localusers defined for vdomain,
    # we should grab them otherwise they'll be missed
    foreach (@{$config{'localusers'}}) {
@@ -554,8 +694,11 @@ sub allusers {
    }
 
    foreach $logindomain (@domains) {
+
+      # REINIT %config for auth_module as each domain may use different auth_module!
+
       %config_raw=();
-      load_owconf(\%config_raw, "/var/www/conf/openwebmail/openwebmail.conf.default");
+      load_owconf(\%config_raw, "/var/www/conf/openwebmail/defaults/openwebmail.conf");
       if ( -f "/var/www/conf/openwebmail/openwebmail.conf") {
          read_owconf(\%config, \%config_raw, "/var/www/conf/openwebmail/openwebmail.conf");
          print "D readconf /var/www/conf/openwebmail/openwebmail.conf\n" if ($opt{'debug'});
@@ -630,7 +773,7 @@ sub usertool {
       $>=$euid_to_use;
 
       %config_raw=();
-      load_owconf(\%config_raw, "/var/www/conf/openwebmail/openwebmail.conf.default");
+      load_owconf(\%config_raw, "/var/www/conf/openwebmail/defaults/openwebmail.conf");
       if ( -f "/var/www/conf/openwebmail/openwebmail.conf") {
          read_owconf(\%config, \%config_raw, "/var/www/conf/openwebmail/openwebmail.conf");
          print "D readconf /var/www/conf/openwebmail/openwebmail.conf\n" if ($opt{'debug'});
@@ -653,7 +796,7 @@ sub usertool {
       $logindomain=lc(safedomainname($logindomain));
       print "D loginuser=$loginuser, logindomain=$logindomain\n" if ($opt{'debug'});
 
-      if (defined($config{'domainname_equiv'}{'map'}{$logindomain})) {
+      if (defined $config{'domainname_equiv'}{'map'}{$logindomain}) {
          $logindomain=$config{'domainname_equiv'}{'map'}{$logindomain};
          print "D logindomain equiv to $logindomain\n" if ($opt{'debug'});
       }
@@ -686,14 +829,10 @@ sub usertool {
          print "D system user $user, skipped!\n" if ($opt{'debug'});
          next;
       }
-      if (defined($homedir_processed{$homedir})) {
-         print "D $loginname homedir already processed, skipped!\n" if ($opt{'debug'});
-         next;
-      }
-      $homedir_processed{$homedir}=1;
 
       if ( $>!=$uuid &&
            $>!=0 &&	# setuid root is required if spool is located in system dir
+           !$config{'use_homedirspools'} &&
           ($config{'mailspooldir'} eq "/var/mail" ||
            $config{'mailspooldir'} eq "/var/spool/mail")) {
          print "This operation is only available to root\n"; openwebmail_exit(0);
@@ -717,6 +856,7 @@ sub usertool {
          print "D change realname to $userrealname\n" if ($opt{'debug'});
       }
 
+      my $syshomedir=$homedir;	# keep this for later use
       my $owuserdir = ow::tool::untaint("$config{'ow_usersdir'}/".($config{'auth_withdomain'}?"$domain/$user":$user));
       if ( !$config{'use_syshomedir'} ) {
          $homedir = $owuserdir;
@@ -726,6 +866,12 @@ sub usertool {
          print "D homedir is /, skipped!\n" if ($opt{'debug'});
       }
 
+      if (defined $homedir_processed{$homedir}) {
+         print "D $loginname homedir already processed, skipped!\n" if ($opt{'debug'});
+         next;
+      }
+      $homedir_processed{$homedir}=1;
+
       $user=ow::tool::untaint($user);
       $uuid=ow::tool::untaint($uuid);
       $ugid=ow::tool::untaint($ugid);
@@ -737,7 +883,7 @@ sub usertool {
             my $domainhome=ow::tool::untaint("$config{'ow_usersdir'}/$domain");
             if (!-d $domainhome) {
                mkdir($domainhome, 0750) or die("Couldn't create domain homedir $domainhome");
-               my $mailgid=getgrnam('mail');
+               my $mailgid=getgrnam('_mail');
                chown($uuid, $mailgid, $domainhome);
             }
          }
@@ -759,7 +905,7 @@ sub usertool {
 
       umask(0077);
       if ( $>==0 ) { # switch to uuid:mailgid if process is setuid to root
-         my $mailgid=getgrnam('mail');	# for better compatibility with other mail progs
+         my $mailgid=getgrnam('_mail');	# for better compatibility with other mail progs
          ow::suid::set_euid_egids($uuid, $mailgid, split(/\s+/,$ugid));
          if ( $)!~/\b$mailgid\b/) {	# group mail doesn't exist?
             print "Set effective gid to mail($mailgid) failed!"; openwebmail_exit(0);
@@ -767,6 +913,9 @@ sub usertool {
       }
       print "D ruid=$<, euid=$>, rgid=$(, eguid=$)\n" if ($opt{'debug'});
 
+      # locate existing .openwebmail
+      find_and_move_dotdir($syshomedir, $owuserdir) if (!-d dotpath('/'));	# locate existing .openwebmail
+
       # get user release date
       my $user_releasedate=read_releasedatefile();
 
@@ -799,6 +948,7 @@ sub usertool {
             print "D do release upgrade...\n" if ($opt{'debug'});
             update_releasedatefile();
          }
+         update_openwebmailrc($user_releasedate);
       }
       if ( ! -d dotpath('/') ) {
          print "D ".dotpath('/')." doesn't exist\n" if ($opt{'debug'});
@@ -816,12 +966,12 @@ sub usertool {
       %prefs = readprefs();
 
       if ($opt{'pop3'}) {
-         my $ret=getpop3s($POP3_TIMEOUT);
-         print "getpop3s($POP3_TIMEOUT) return $ret\n" if (!$opt{'quiet'} && $ret!=0);
+         my $ret=pop3_fetches($POP3_TIMEOUT);
+         print "pop3_fetches($POP3_TIMEOUT) return $ret\n" if (!$opt{'quiet'} && $ret!=0);
       }
       if ($opt{'zap'}) {
-         my $ret=cleantrash();
-         print "cleantrash() return $ret\n" if (!$opt{'quiet'} && $ret!=0);
+         my $ret=clean_trash_spamvirus();
+         print "clean_trash_spamvirus() return $ret\n" if (!$opt{'quiet'} && $ret!=0);
       }
 
       if ($opt{'ir'}) {
@@ -857,6 +1007,16 @@ sub usertool {
          my $ret=checknotify();
          print "checknotify() return $ret\n" if (!$opt{'quiet'} && $ret!=0);
       }
+      if ($opt{'unlock'}) {
+         my $ret=unlockfiles();
+         print "unlockfiles() return $ret\n" if (!$opt{'quiet'} && $ret!=0);
+      }
+      if ($opt{'convert_addressbooks'}) {
+         loadlang($prefs{'language'});	# for converted filename $lang_text{abook_converted}
+         print "converting user $user addressbook..." if (!$opt{'quiet'});
+         my $ret=convert_addressbook('user', $prefs{'charset'});
+         print "done.\n" if (!$opt{'quiet'});
+      }
 
       $usercount++;
    }
@@ -900,7 +1060,7 @@ sub folderindex {
          }
          @messageids=get_messageids_sorted_by_offset($folderdb);
 
-         if (!ow::filelock::lock($folderfile, LOCK_SH)) {
+         if (!ow::filelock::lock($folderfile, LOCK_SH|LOCK_NB)) {
             print "Couldn't get read lock on $folderfile\n" if (!$opt{'quiet'});
             return -1;
          }
@@ -913,13 +1073,15 @@ sub folderindex {
 
          for(my $i=0; $i<=$#messageids; $i++) {
             @attr=get_message_attributes($messageids[$i], $folderdb);
+            next if ($#attr<0);        # msg not found in db
+
             $headerlen=$attr[$_HEADERSIZE]||6;
             seek(FOLDER, $attr[$_OFFSET], 0);
             read(FOLDER, $buff, $headerlen);
             seek(FOLDER, $attr[$_OFFSET]+$attr[$_SIZE], 0);
             read(FOLDER, $buff2, 6);
 
-            my @check=(0,0,0); 
+            my @check=(0,0,0);
             $check[0]=1 if ( $buff=~/^From / );
             $check[1]=1 if ( $attr[$_HEADERCHKSUM] eq ow::tool::calc_checksum(\$buff) );
             $check[2]=1 if ( $buff2=~/^From / || $buff2 eq "" );
@@ -927,22 +1089,26 @@ sub folderindex {
             $error++ if (!$check[0] || !$check[1] || !$check[2]);
 
             printf ("%s%s%s %4d, OFFSET:%8d, SIZE:%8d, HSIZE:%4d, DATE:%s, CHARSET:%s, STAT:%3s, MSGID:%s, FROM:%s, TO:%s, SUB:%s\n",
-                    $check[0]?'+':'-', $check[1]?'+':'-', $check[2]?'+':'-', 
+                    $check[0]?'+':'-', $check[1]?'+':'-', $check[2]?'+':'-',
                     $i+1, $attr[$_OFFSET], $attr[$_SIZE], $attr[$_HEADERSIZE], $attr[$_DATE], $attr[$_CHARSET], $attr[$_STATUS],
                     substr($messageids[$i],0,50), $attr[$_FROM], $attr[$_TO], $attr[$_SUBJECT]) if (!$opt{'quiet'});
             #printf ("buf=$buff, buff2=$buff2\n");
 
             $folderinfo{'ALLMESSAGES'}++;
-            $folderinfo{'NEWMESSAGES'}++ if ($attr[$_STATUS]!~/r/i);
-            $folderinfo{'INTERNALMESSAGES'}++ if (is_internal_subject($attr[$_SUBJECT]));
-            $folderinfo{'ZAPSIZE'}+=$attr[$_SIZE] if ($attr[$_STATUS]=~/Z/i);
+            if ($attr[$_STATUS]=~/Z/i) {
+               $folderinfo{'ZAPMESSAGES'}++; $folderinfo{'ZAPSIZE'}+=$attr[$_SIZE];
+            } elsif (is_internal_subject($attr[$_SUBJECT])) {
+               $folderinfo{'INTERNALMESSAGES'}++; $folderinfo{'INTERNALSIZE'}+=$attr[$_SIZE];
+            } elsif ($attr[$_STATUS]!~/R/i) {
+               $folderinfo{'NEWMESSAGES'}++;
+            }
          }
 
          $folderinfo{'DBVERSION'}=$DBVERSION;
          $folderinfo{'METAINFO'}=ow::tool::metainfo($folderfile);
          if (!$opt{'quiet'}) {
             print "\n";
-            foreach my $key (qw(DBVERSION METAINFO ALLMESSAGES NEWMESSAGES INTERNALMESSAGES ZAPSIZE)) {
+            foreach my $key (qw(DBVERSION METAINFO ALLMESSAGES NEWMESSAGES INTERNALMESSAGES INTERNALSIZE ZAPMESSAGES ZAPSIZE)) {
                my $sign="+++";
                if ($FDB{$key} ne $folderinfo{$key}) {
                   $sign="---"; $error++;
@@ -957,7 +1123,7 @@ sub folderindex {
          print "$error errors in db $folderdb\n" if (!$opt{'quiet'});
 
       } elsif ($op eq "zap") {
-         if (!ow::filelock::lock($folderfile, LOCK_EX|LOCK_NB)) {
+         if (!ow::filelock::lock($folderfile, LOCK_EX)) {
             print "Couldn't get write lock on $folderfile\n" if (!$opt{'quiet'});
             next;
          }
@@ -971,7 +1137,7 @@ sub folderindex {
          ow::filelock::lock($folderfile, LOCK_UN);
 
       } else {
-         if (!ow::filelock::lock($folderfile, LOCK_EX|LOCK_NB)) {
+         if (!ow::filelock::lock($folderfile, LOCK_EX)) {
             print "Couldn't get write lock on $folderfile\n" if (!$opt{'quiet'});
             next;
          }
@@ -987,7 +1153,7 @@ sub folderindex {
                ow::filelock::lock($folderfile, LOCK_UN);
                next;
             }
-            $FDB{'METAINFO'}="ERR";
+            @FDB{'METAINFO', 'LSTMTIME'}=('ERR', -1);
             ow::dbm::close(\%FDB, $folderdb);
             $ret=update_folderindex($folderfile, $folderdb);
          }
@@ -1008,16 +1174,29 @@ sub folderindex {
 }
 
 
-sub cleantrash {
-   my ($trashfile, $trashdb)=get_folderpath_folderdb($user, 'mail-trash');
-   if (ow::filelock::lock($trashfile, LOCK_EX|LOCK_NB)) {
-      my $deleted=delete_message_by_age($prefs{'trashreserveddays'}, $trashdb, $trashfile);
-      if ($deleted >0) {
-         writelog("clean trash - delete $deleted msgs from mail-trash");
-         writehistory("clean trash - delete $deleted msgs from mail-trash");
-         print "$deleted msgs deleted from mail-trash\n" if (!$opt{'quiet'});
+sub clean_trash_spamvirus {
+   my %reserveddays=('mail-trash' => $prefs{'trashreserveddays'},
+                     'spam-mail'  => $prefs{'spamvirusreserveddays'},
+                     'virus-mail' => $prefs{'spamvirusreserveddays'} );
+   my (@f, $msg);
+   push(@f, 'virus-mail') if ($config{'has_virusfolder_by_default'});
+   push(@f, 'spam-mail') if ($config{'has_spamfolder_by_default'});
+   push(@f, 'mail-trash');
+   foreach my $folder (@f) {
+      my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $folder);
+      if (ow::filelock::lock($folderfile, LOCK_EX)) {
+         my $deleted=delete_message_by_age($reserveddays{$folder}, $folderdb, $folderfile);
+         if ($deleted > 0) {
+            $msg.=', ' if ($msg ne '');
+            $msg.="$deleted msg deleted from $folder";
+         }
+         ow::filelock::lock($folderfile, LOCK_UN);
       }
-      ow::filelock::lock($trashfile, LOCK_UN);
+   }
+   if ($msg ne '') {
+      writelog("clean trash/spam/virus - $msg");
+      writehistory("clean trash/spam/virus - $msg");
+      print "clean trash/spam/virus - $msg\n" if (!$opt{'quiet'});
    }
    return 0;
 }
@@ -1074,10 +1253,9 @@ sub checknewmail {
          my $login=$user;  $login.="\@$domain" if ($config{'auth_withdomain'});
          my ($pop3ssl, $pop3passwd, $pop3del)
 		=(split(/\@\@\@/, $accounts{"$config{'authpop3_server'}:$config{'authpop3_port'}\@\@\@$login"}))[2,4,5];
-         my ($ret, $errmsg) = ow::pop3::fetchmail($config{'authpop3_server'},$config{'authpop3_port'}, $pop3ssl,
-                                                  $login, $pop3passwd, $pop3del,
-                                                  dotpath("uidl.$login\@$config{'authpop3_server'}"), $spoolfile,
-                                                  $config{'deliver_use_GMT'}, $prefs{'daylightsaving'});
+
+         my ($ret, $errmsg) = fetchmail($config{'authpop3_server'}, $config{'authpop3_port'}, $pop3ssl,
+                                        $login, $pop3passwd, $pop3del);
          if ($ret<0) {
             writelog("pop3 error - $errmsg at $login\@$config{'authpop3_server'}:$config{'authpop3_port'}");
          }
@@ -1091,35 +1269,19 @@ sub checknewmail {
 
    update_folderindex($spoolfile, $folderdb);
 
-   my ($filtered, $r_filtered)=filtermessage($user, 'INBOX', \%prefs);
-   if ($filtered==-10) {	# check again if abort due to db inconsistence
-      ($filtered, $r_filtered)=filtermessage($user, 'INBOX', \%prefs);
-   }
-
-   if ($filtered>0) {
-      my $dststr;
-      foreach my $destination (sort keys %{$r_filtered}) {
-         next if ($destination eq '_ALL' || $destination eq 'INBOX');
-         $dststr .= ", " if ($dststr ne "");
-         $dststr .= $destination;
-         $dststr .= "(${$r_filtered}{$destination})" if (${$r_filtered}{$destination} ne $filtered);
-      }
-      writelog("filter message - filter $filtered msgs from INBOX to $dststr");
-      writehistory("filter message - filter $filtered msgs from INBOX to $dststr");
-   }
+   # filtermessage in background
+   filtermessage($user, 'INBOX', \%prefs);
 
    if (!$opt{'quiet'}) {
-      my (%FDB, $allmessages, $internalmessages, $newmessages);
+      my %FDB;
       if (!ow::dbm::open(\%FDB, $folderdb, LOCK_SH)) {
          print "couldn't get read lock on db $folderdb\n";
          return -1;
       }
-      $allmessages=$FDB{'ALLMESSAGES'};
-      $internalmessages=$FDB{'INTERNALMESSAGES'};
-      $newmessages=$FDB{'NEWMESSAGES'};
+      my $newmessages=$FDB{'NEWMESSAGES'};
+      my $oldmessages=$FDB{'ALLMESSAGES'}-$FDB{'ZAPMESSAGES'}-$FDB{'INTERNALMESSAGES'}-$newmessages;
       ow::dbm::close(\%FDB, $folderdb);
 
-      my $oldmessages=$allmessages-$internalmessages;
       if ($newmessages == 1 ) {
          print "has 1 new mail\n";
       } elsif ($newmessages > 1 ) {
@@ -1161,19 +1323,15 @@ sub checknewevent {
       }
    }
 
-   my ($easter_month, $easter_day) = ow::datetime::gregorian_easter($year); # compute once
-
    my @indexlist=();
-   push(@indexlist, @{$indexes{$date}}) if (defined($indexes{$date}));
-   push(@indexlist, @{$indexes{'*'}})   if (defined($indexes{'*'}));
+   push(@indexlist, @{$indexes{$date}}) if (defined $indexes{$date});
+   push(@indexlist, @{$indexes{'*'}})   if (defined $indexes{'*'});
    @indexlist=sort { ($items{$a}{'starthourmin'}||1E9)<=>($items{$b}{'starthourmin'}||1E9) } @indexlist;
 
    for my $index (@indexlist) {
       if ($date=~/$items{$index}{'idate'}/ ||
           $date2=~/$items{$index}{'idate'}/ ||
-          ow::datetime::easter_match($year,$month,$day,
-                                     $easter_month,$easter_day,
-                                     $items{$index}{'idate'}) ) {
+          ow::datetime::easter_match($year,$month,$day,$items{$index}{'idate'}) ) {
          if ($items{$index}{'starthourmin'}>=$hourmin ||
              $items{$index}{'endhourmin'}>$hourmin ||
              $items{$index}{'starthourmin'}==0) {
@@ -1241,11 +1399,9 @@ sub checknotify {
       }
    }
 
-   my ($easter_month, $easter_day) = ow::datetime::gregorian_easter($year); # compute once
-
    my @indexlist=();
-   push(@indexlist, @{$indexes{$date}}) if (defined($indexes{$date}));
-   push(@indexlist, @{$indexes{'*'}})   if (defined($indexes{'*'}));
+   push(@indexlist, @{$indexes{$date}}) if (defined $indexes{$date});
+   push(@indexlist, @{$indexes{'*'}})   if (defined $indexes{'*'});
    @indexlist=sort { ($items{$a}{'starthourmin'}||1E9)<=>($items{$b}{'starthourmin'}||1E9) } @indexlist;
 
    my $future_items=0;
@@ -1253,28 +1409,26 @@ sub checknotify {
       if ( $items{$index}{'email'} &&
            ($date=~/$items{$index}{'idate'}/  ||
             $date2=~/$items{$index}{'idate'}/ ||
-            ow::datetime::easter_match($year,$month,$day,
-                                       $easter_month,$easter_day,
-                                       $items{$index}{'idate'})) ) {
+            ow::datetime::easter_match($year,$month,$day,$items{$index}{'idate'})) ) {
          if ( ($items{$index}{'starthourmin'}>=$checkstart &&
                $items{$index}{'starthourmin'}<$checkend) ||
               ($items{$index}{'starthourmin'}==0 &&
                $checkstart eq "0000") ) {
             my $itemstr;
             if ($items{$index}{'starthourmin'}==0) {
-               $itemstr="##:##";
+               $itemstr="##:##\n";
             } elsif ($items{$index}{'endhourmin'}==0) {
-               $itemstr=hourmin($items{$index}{'starthourmin'});
+               $itemstr=hourmin($items{$index}{'starthourmin'})."\n";
             } else {
-               $itemstr=hourmin($items{$index}{'starthourmin'})."-".hourmin($items{$index}{'endhourmin'});
+               $itemstr=hourmin($items{$index}{'starthourmin'})."-".hourmin($items{$index}{'endhourmin'})."\n";
             }
-            $itemstr .= "  $items{$index}{'string'}";
-            $itemstr .= " ($items{$index}{'link'})" if ($items{$index}{'link'});
+            $itemstr.=(iconv($items{$index}{'charset'}, $prefs{'charset'}, $items{$index}{'string'}))[0]."\n";
+            $itemstr.=$items{$index}{'link'}."\n" if ($items{$index}{'link'});
 
-            if (defined($message{$items{$index}{'email'}})) {
-               $message{$items{$index}{'email'}} .= $itemstr."\n";
+            if (defined $message{$items{$index}{'email'}}) {
+               $message{$items{$index}{'email'}} .= $itemstr;
             } else {
-               $message{$items{$index}{'email'}} = $itemstr."\n";
+               $message{$items{$index}{'email'}} = $itemstr;
             }
          }
          if ($items{$index}{'starthourmin'}>=$checkend) {
@@ -1290,11 +1444,10 @@ sub checknotify {
       }
    }
 
-   my $title=$prefs{'dateformat'}||"mm/dd/yyyy";
    my ($m, $d)=(sprintf("%02d",$month), sprintf("%02d",$day));
+   my $title=$prefs{'dateformat'}||"mm/dd/yyyy";
    $title=~s/yyyy/$year/; $title=~s/mm/$m/; $title=~s/dd/$d/;
-   $title.=" Event(s) between ".hourmin($checkstart)."-".hourmin($checkend)."\n".
-           "------------------------------------------------------------\n";
+   $title.=" (".hourmin($checkstart)."-".hourmin($checkend).")\n\n";
    my $from=$prefs{'email'};
    my %userfrom=get_userfrom($logindomain, $loginuser, $user, $userrealname, dotpath('from.book'));
    my $realname=$userfrom{$from};
@@ -1335,7 +1488,7 @@ sub send_mail {
                         Port => $config{'smtpport'},
                         Timeout => 120,
                         Hello => ${$config{'domainnames'}}[0]) or
-      die "Couldn't SMTP server $config{'smtpserver'}:$config{'smtpport'}!";
+      die "Couldn't connect SMTP server $config{'smtpserver'}:$config{'smtpport'}!";
 
    # SMTP SASL authentication (PLAIN only)
    if ($config{'smtpauth'}) {
@@ -1363,13 +1516,10 @@ sub send_mail {
                    "To: ".ow::mime::encode_mimewords($to, ('Charset'=>$prefs{'charset'}))."\n");
    $smtp->datasend("Reply-To: ".ow::mime::encode_mimewords($prefs{'replyto'}, ('Charset'=>$prefs{'charset'}))."\n") if ($prefs{'replyto'});
 
-   my $xmailer = $config{'name'};
-   $xmailer .= " $config{'version'} $config{'releasedate'}" if ($config{'xmailer_has_version'});
-
    $smtp->datasend("Subject: ".ow::mime::encode_mimewords($subject, ('Charset'=>$prefs{'charset'}))."\n",
                    "Date: $date\n",
                    "Message-Id: $fakedid\n",
-                   "X-Mailer: $xmailer\n",
+                   safexheaders($config{'xheaders'}),
                    "MIME-Version: 1.0\n",
                    "Content-Type: text/plain; charset=$prefs{'charset'}\n\n",
                    $body, "\n\n");
@@ -1385,16 +1535,15 @@ sub send_mail {
    return 0;
 }
 
-sub getpop3s {
+sub pop3_fetches {
    my $timeout=$_[0];
    my ($spoolfile, $header)=get_folderpath_folderdb($user, 'INBOX');
-   my (%accounts, $response);
-
    # create system spool file /var/mail/xxxx
    if ( ! -f "$spoolfile" ) {
       open (F, ">>$spoolfile"); close(F);
    }
 
+   my %accounts=();
    my $pop3bookfile=dotpath('pop3.book');
 
    return 0 if (!-f $pop3bookfile);
@@ -1412,10 +1561,8 @@ sub getpop3s {
       }
       next if ($disallowed);
 
-      my ($ret, $errmsg) = ow::pop3::fetchmail($pop3host, $pop3port, $pop3ssl,
-                                               $pop3user, $pop3passwd, $pop3del,
-                                               dotpath("uidl.$pop3user\@$pop3host"), $spoolfile,
-                                               $config{'deliver_use_GMT'}, $prefs{'daylightsaving'});
+      my ($ret, $errmsg) = fetchmail($pop3host, $pop3port, $pop3ssl,
+                                     $pop3user, $pop3passwd, $pop3del);
       if ( $ret<0) {
          writelog("pop3 error - $errmsg at $pop3user\@$pop3host:$pop3port");
       }
@@ -1423,3 +1570,44 @@ sub getpop3s {
 
    return 0;
 }
+
+sub unlockfiles {
+   my @cmd;
+   my $lsofbin=ow::tool::findsbin('lsof')||ow::tool::findbin('lsof');
+   if ($lsofbin eq '') {
+      print "Program lsof not found, please install lsof first";
+      return 1;
+   }
+
+   my $folderdir=ow::tool::untaint("$homedir/$config{'homedirfolderdirname'}");
+   my $dbdir=ow::tool::untaint(dotpath('/'));
+   my $spooldir=(get_folderpath_folderdb($user, 'INBOX'))[0]; $spooldir=~s!(.*)/.*!$1!;
+
+   push(@cmd, $lsofbin, '+w', '-l', '-S2', '-Di');
+   foreach ($spooldir, $folderdir, $dbdir, $config{'ow_etcdir'}) {
+      push(@cmd, '+d'.$_);
+   }
+   push(@cmd, '-a', '-d^cwd');
+
+   # since lsof read/write tmp cache file with ruid no matter what euid is
+   # so we set euid=root or euid won't have enough privilege to read/write lsof cache file
+   my $euid=$>; $>=0 if ($<==0);
+   my ($stdout, $stderr, $exit, $sig)=ow::execute::execute(@cmd);
+   $>=$euid if ($<==0);
+
+   my @pids;
+   foreach (split(/\n/, $stdout)) {
+      my ($cmd, $pid, $puid, $fd, $type, $device, $size, $mode, $fname)=split(/\s+/);
+      next if ($puid!=$euid);
+      next if ($fd!~/\d+([A-Za-z])([A-Za-z])/);
+      my ($rw, $lock)=($1, $2);
+      print "program=$cmd, pid=$pid, lock=$lock, file=$fname\n";
+      push(@pids, ow::tool::untaint($pid));
+   }
+   if ($#pids>=0) {
+      print "\nKill above processes to remove filelock for $user? (y/N) ";
+      $_=<STDIN>;
+      kill 9, @pids if (/y/i);
+   }
+   return 0;
+}
