@@ -2,8 +2,8 @@
 #
 # openwebmail-vdomain.pl - virtual domain user management
 #
-# 2003/02/26 Bernd Bass, owm@adminsquare.de
-# 2003/05/09 Scott Mazur, scott@littlefish.ca
+# 2003/02/26 Bernd Bass, owm.AT.adminsquare.de
+# 2003/05/09 Scott Mazur, scott.AT.littlefish.ca
 #
 
 #
@@ -101,14 +101,14 @@
 #
 
 use vars qw($SCRIPT_DIR);
-if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1 }
+if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { local $1; $SCRIPT_DIR=$1 }
 if ($SCRIPT_DIR eq '' && open(F, '/var/www/conf/openwebmail/openwebmail_path.conf')) {
-   $_=<F>; close(F); if ( $_=~/^(\S*)/) { $SCRIPT_DIR=$1 }
+   $_=<F>; close(F); if ( $_=~/^(\S*)/) { local $1; $SCRIPT_DIR=$1 }
 }
 if ($SCRIPT_DIR eq '') { print "Content-type: text/html\n\nSCRIPT_DIR not set in /var/www/conf/openwebmail/openwebmail_path.conf !\n"; exit 0; }
 push (@INC, $SCRIPT_DIR);
 
-foreach (qw(PATH ENV BASH_ENV CDPATH IFS TERM)) { $ENV{$_}='' }	# secure ENV
+foreach (qw(ENV BASH_ENV CDPATH IFS TERM)) {delete $ENV{$_}}; $ENV{PATH}='/bin:/usr/bin'; # secure ENV
 umask(0002); # make sure the openwebmail group can write
 
 use strict;
@@ -128,6 +128,9 @@ require "auth/auth.pl";
 require "quota/quota.pl";
 require "shares/ow-shared.pl";
 
+# optional module
+ow::tool::has_module('Compress/Zlib.pm');
+
 # common globals
 use vars qw(%config %config_raw);
 use vars qw($thissession);
@@ -161,6 +164,7 @@ foreach ("$config{'vdomain_vmpop3_pwdpath'}/$domain/$config{'vdomain_vmpop3_pwdn
 }
 
 my $action = param('action');
+writelog("debug - request vdomain begin, action=$action - " .__FILE__.":". __LINE__) if ($config{'debug_request'});
 if ($action eq 'display_vuserlist') {
    display_vuserlist();
 } elsif ($action eq 'edit_vuser' ||
@@ -174,6 +178,7 @@ if ($action eq 'display_vuserlist') {
 } else {
    openwebmailerror(__FILE__, __LINE__, "Action $lang_err{'has_illegal_chars'}");
 }
+writelog("debug - request vdomain end, action=$action - " .__FILE__.":". __LINE__) if ($config{'debug_request'});
 
 openwebmail_requestend();
 ########## END MAIN ##############################################
@@ -473,14 +478,18 @@ sub edit_vuser {
       my $val=$from_list{$_};$val=~s/'/\\'/;
       my $txt=$_;
       $txt .= " ($lang_text{'email_alias'})" if (/\@$domain$/);
-      $temphtml .= qq|<tr bgcolor=$bgcolor><td><a href="Javascript:Update('$key','$val')">$txt</a></td>| .
-                               qq|<td>$from_list{$_}</td>| .
-                               qq|<td align="center">| .
-                               submit(-name=>'aliasdel_button',
-                                  -value=>$lang_text{'delete'},
-                                  -onClick=>"Delete('$key')",
-                                  -class=>'medtext') .
-                               "</td></tr>\n";
+      $temphtml .= qq|<tr bgcolor=$bgcolor |.
+                   qq|onMouseOver='this.style.backgroundColor=$style{tablerow_hicolor};' |.
+                   qq|onMouseOut='this.style.backgroundColor=$bgcolor;' |.
+                   qq|>\n|.
+                   qq|<td><a href="Javascript:Update('$key','$val')">$txt</a></td>|.
+                   qq|<td>$from_list{$_}</td>|.
+                   qq|<td align="center">|.
+                   submit(-name=>'aliasdel_button',
+                          -value=>$lang_text{'delete'},
+                          -onClick=>"Delete('$key')",
+                          -class=>'medtext') .
+                   qq|</td></tr>\n|;
       if ($bgcolor eq $style{"tablerow_dark"}) {
          $bgcolor = $style{"tablerow_light"};
       } else {
@@ -573,7 +582,7 @@ sub change_vuser {
    if ( $emailkey ) {
       if ( $alias ne $emailkey and $alias ne $vuser) {
          # alias entry needs additional edit checks.
-         if ( ! defined($from_list{$emailkey}) and $config{'vdomain_maxalias'} < ($#alias_list + 2) ) {
+         if ( ! defined $from_list{$emailkey} and $config{'vdomain_maxalias'} < ($#alias_list + 2) ) {
             $alert=$lang_err{'vdomain_toomanyalias'};
          } elsif ( valias_list_exists($vuser,$alias) ) {
              $alert="$alias\@$domain $lang_err{'already_exists'}";
@@ -639,7 +648,7 @@ sub change_vuser {
       return;
    }
 
-   my $vgid=getgrnam('mail');	# for better compatibility with other mail progs
+   my $vgid=getgrnam('_mail');	# for better compatibility with other mail progs
 
    if ( $new ) { # CREATE NEW USER
       my $aliastxt='';
@@ -658,13 +667,13 @@ sub change_vuser {
          if (mkdir ($vhomedir, 0700) && chown($vuid, $vgid, $vhomedir)) {
             writelog("vdomain $user: $vuser\@$domain  create homedir - $vhomedir, uid=$vuid, gid=$vgid");
          } else {
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'cant_create_dir'} $vhomedir ($!)");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_create'} $vhomedir ($!)");
          }
       }
 
       # switch to virtual user
       ow::suid::set_euid_egids($vuid,$vgid);
-      
+
       # create dot directory structure
       check_and_create_dotdir(_dotpath('/', $domain, $vuser, $vhomedir));
 
@@ -672,7 +681,7 @@ sub change_vuser {
       my $releasedatefile=_dotpath('release.date', $domain, $vuser, $vhomedir);
       writelog("vdomain $user: $vuser\@$domain  create release.date - $releasedatefile, uid=$vuid, gid=$vgid");
       open(RD, ">$releasedatefile") or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $releasedatefile ($!)");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_write'} $releasedatefile ($!)");
       print RD "$config{'releasedate'}\n";
       close(RD);
       chmod(0700, $releasedatefile);
@@ -681,7 +690,7 @@ sub change_vuser {
       my $dotforward="$vhomedir/.forward";
       writelog("vdomain $user: $vuser\@$domain  create .forward - $dotforward, uid=$vuid, gid=$vgid");
       open (DF, ">$dotforward") or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $dotforward ($!)");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_write'} $dotforward ($!)");
       print DF vdomain_userspool($vuser, $vhomedir)."\n";
       close (DF);
       chmod(0700, $dotforward);
@@ -732,7 +741,7 @@ sub change_vuser {
          # changed froms?
          if ($orig_realnm eq $realnm) {
             foreach (keys %from_list) {
-               if (! defined( $orig_from_list{$_} ) or $orig_from_list{$_} ne $from_list{$_} ) {
+               if (!defined $orig_from_list{$_} or $orig_from_list{$_} ne $from_list{$_} ) {
                   $match=0;
                   last;
                }
@@ -1011,7 +1020,7 @@ sub from_list {
 
    # add in the email aliases
    foreach ( valias_list($vuser) ){
-      $fromlist{"$_\@$domain"}='' if ( ! defined( $fromlist{"$_\@$domain"} ));
+      $fromlist{"$_\@$domain"}='' if (!defined $fromlist{"$_\@$domain"});
    }
    return $realnm,%fromlist;
 }
@@ -1071,7 +1080,7 @@ sub vuser_update {
    }
 
    open ($fh, ">$file") or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $file ($!)");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_write'} $file ($!)");
 
    print $fh @lines;
 
@@ -1103,9 +1112,9 @@ sub from_update {
    else { writelog("vdomain $user: $vuser\@$domain  create from.book - $frombook, uid=$<, gid=$>"); }
 
    ow::filelock::lock($frombook, LOCK_EX) or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $frombook");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} $frombook");
    open (FB, ">$frombook") or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $frombook ($!)");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_write'} $frombook ($!)");
 
    print FB "$vuser\@$domain\@\@\@$realnm\n" if ($realnm);
    foreach (sort keys %from_list) {
@@ -1146,7 +1155,7 @@ sub valias_update {
    }
 
    open ($fh, ">$file") or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $file ($!)");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_write'} $file ($!)");
 
    print $fh @lines;
 
@@ -1215,7 +1224,7 @@ sub vpasswd_update {
    writelog("vdomain $user: $vuser\@$domain  disable user login") if ($disable);
 
    open ($fh, ">$file") or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $file ($!)");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_write'} $file ($!)");
 
    print $fh @lines;
 
@@ -1294,13 +1303,13 @@ sub root_open {
 
    if ($action ne '') {
       ow::filelock::lock($file, LOCK_EX) or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $file");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} $file");
    } else {
       ow::filelock::lock($file, LOCK_SH|LOCK_NB) or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $file");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_readlock'} $file");
    }
    open ($fh, "$action$file") or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $file ($!)");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_read'} $file ($!)");
 
    return ($fh, $file, $origruid, $origeuid, $origegid);
 }
