@@ -4,14 +4,14 @@
 #
 
 use vars qw($SCRIPT_DIR);
-if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { $SCRIPT_DIR=$1 }
+if ( $0 =~ m!^(\S*)/[\w\d\-\.]+\.pl! ) { local $1; $SCRIPT_DIR=$1 }
 if ($SCRIPT_DIR eq '' && open(F, '/var/www/conf/openwebmail/openwebmail_path.conf')) {
-   $_=<F>; close(F); if ( $_=~/^(\S*)/) { $SCRIPT_DIR=$1 }
+   $_=<F>; close(F); if ( $_=~/^(\S*)/) { local $1; $SCRIPT_DIR=$1 }
 }
 if ($SCRIPT_DIR eq '') { print "Content-type: text/html\n\nSCRIPT_DIR not set in /var/www/conf/openwebmail/openwebmail_path.conf !\n"; exit 0; }
 push (@INC, $SCRIPT_DIR);
 
-foreach (qw(PATH ENV BASH_ENV CDPATH IFS TERM)) { $ENV{$_}='' }	# secure ENV
+foreach (qw(ENV BASH_ENV CDPATH IFS TERM)) {delete $ENV{$_}}; $ENV{PATH}='/bin:/usr/bin'; # secure ENV
 umask(0002); # make sure the openwebmail group can write
 
 use strict;
@@ -29,6 +29,7 @@ require "modules/datetime.pl";
 require "modules/lang.pl";
 require "modules/mime.pl";
 require "modules/mailparse.pl";
+require "modules/tnef.pl";
 require "modules/htmltext.pl";
 require "modules/htmlrender.pl";
 require "modules/execute.pl";
@@ -37,6 +38,10 @@ require "quota/quota.pl";
 require "shares/ow-shared.pl";
 require "shares/iconv.pl";
 require "shares/maildb.pl";
+require "shares/lockget.pl";
+
+# optional module
+ow::tool::has_module('Compress/Zlib.pm');
 
 # common globals
 use vars qw(%config %config_raw);
@@ -57,16 +62,13 @@ use vars qw($escapedkeyword);
 
 ########## MAIN ##################################################
 openwebmail_requestbegin();
-$SIG{PIPE}=\&openwebmail_exit;	# for user stop
-$SIG{TERM}=\&openwebmail_exit;	# for user stop
-
 userenv_init();
 
 if (!$config{'enable_webmail'}) {
    openwebmailerror(__FILE__, __LINE__, "$lang_text{'webmail'} $lang_err{'access_denied'}");
 }
 
-$folder = param('folder') || 'INBOX';
+$folder = ow::tool::unescapeURL(param('folder')) || 'INBOX';
 $page = param('page') || 1;
 $sort = param('sort') || $prefs{'sort'} || 'date';
 $keyword = param('keyword') || '';
@@ -75,6 +77,7 @@ $searchtype = param('searchtype') || 'subject';
 $escapedkeyword = ow::tool::escapeURL($keyword);
 
 my $action = param('action')||'';
+writelog("debug - request viewatt begin, action=$action - " .__FILE__.":". __LINE__) if ($config{'debug_request'});
 if ($action eq "viewattachment") {
    viewattachment();
 } elsif ($action eq "saveattachment" && $config{'enable_webdisk'}) {
@@ -86,6 +89,7 @@ if ($action eq "viewattachment") {
 } else {
    openwebmailerror(__FILE__, __LINE__, "Action $lang_err{'has_illegal_chars'}");
 }
+writelog("debug - request viewatt end, action=$action - " .__FILE__.":". __LINE__) if ($config{'debug_request'});
 
 openwebmail_requestend();
 ########## END MAIN ##############################################
@@ -126,26 +130,17 @@ sub getattachment {
    my ($folder, $messageid, $nodeid, $wordpreview)=@_;
    my ($folderfile, $folderdb)=get_folderpath_folderdb($user, $folder);
    my $folderhandle=do { local *FH };
+   my ($msgsize, $errmsg, $block);
 
-   ow::filelock::lock($folderfile, LOCK_SH|LOCK_NB) or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $folderfile!");
-   if (update_folderindex($folderfile, $folderdb)<0) {
-      ow::filelock::lock($folderfile, LOCK_UN);
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} $folderdb");
-   }
-   open($folderhandle, "$folderfile");
-   my $r_block= get_message_block($messageid, $folderdb, $folderhandle);
-   close($folderhandle);
-   ow::filelock::lock($folderfile, LOCK_UN);
-
-   if ( !defined(${$r_block}) ) {
+   ($msgsize, $errmsg)=lockget_message_block($messageid, $folderfile, $folderdb, \$block);
+   if ( $msgsize<=0 ) {
       openwebmailerror(__FILE__, __LINE__, "What the heck? Message ".ow::htmltext::str2html($messageid)." seems to be gone!");
    }
 
    my @attr=get_message_attributes($messageid, $folderdb);
    my $convfrom=param('convfrom')||'';
    if ($convfrom eq "") {
-      if ( is_convertable($attr[$_CHARSET], $prefs{'charset'}) ) {
+      if ( is_convertible($attr[$_CHARSET], $prefs{'charset'}) ) {
          $convfrom=lc($attr[$_CHARSET]);
       } else {
          $convfrom='none.prefscharset';
@@ -154,13 +149,10 @@ sub getattachment {
 
    if ( $nodeid eq 'all' ) {
       # return whole msg as an message/rfc822 object
-      my $subject = $attr[$_SUBJECT];
-      if (is_convertable($convfrom, $prefs{'charset'}) ) {
-         ($subject)=iconv($convfrom, $prefs{'charset'}, $subject);
-      }
+      my ($subject) = iconv($convfrom, $prefs{'charset'}, $attr[$_SUBJECT]);
       $subject =~ s/\s+/_/g;
 
-      my $length = length(${$r_block});
+      my $length = length($block);
       my $attheader=qq|Content-Length: $length\n|.
                     qq|Connection: close\n|.
                     qq|Content-Type: message/rfc822; name="$subject.msg"\n|;
@@ -178,12 +170,12 @@ sub getattachment {
                      qq|Cache-Control: private,max-age=900\n|;
       }
 
-      return("$subject.msg", $length, \$attheader, $r_block);
+      return("$subject.msg", $length, \$attheader, \$block);
 
    } else {
       # return a specific attachment
-      my ($header, $body, $r_attachments)=ow::mailparse::parse_rfc822block($r_block, "0", $nodeid);
-      undef(${$r_block}); undef($r_block);
+      my ($header, $body, $r_attachments)=ow::mailparse::parse_rfc822block(\$block, "0", $nodeid);
+      undef($block);
 
       my $r_attachment;
       for (my $i=0; $i<=$#{$r_attachments}; $i++) {
@@ -191,16 +183,13 @@ sub getattachment {
             $r_attachment=${$r_attachments}[$i];
          }
       }
-      if (defined($r_attachment)) {
+      if (defined $r_attachment) {
          my $charset=${$r_attachment}{filenamecharset}||
                      ${$r_attachment}{charset}||
                      $convfrom||
                      $attr[$_CHARSET];
-         if (is_convertable($charset, $prefs{'charset'})) {
-            (${$r_attachment}{filename})=iconv($charset, $prefs{'charset'},
-                                                ${$r_attachment}{filename});
-         }
-
+         my $contenttype = ${$r_attachment}{'content-type'};
+         my $filename = ${$r_attachment}{filename}; $filename=~s/\s$//;
          my $content;
          if (${$r_attachment}{'content-transfer-encoding'} =~ /^base64$/i) {
             $content = decode_base64(${${$r_attachment}{r_content}});
@@ -211,7 +200,20 @@ sub getattachment {
          } else { ## Guessing it's 7-bit, at least sending SOMETHING back! :)
             $content = ${${$r_attachment}{r_content}};
          }
-         if (${$r_attachment}{'content-type'} =~ m#^text/html#i ) {
+
+         if ($contenttype =~ m#^application/ms\-tnef#) {	# try to convery tnef -> zip/tgz/tar
+            my $tnefbin=ow::tool::findbin('tnef');
+            if ($tnefbin ne '') {
+               my ($arcname, $r_arcdata)=ow::tnef::get_tnef_archive($tnefbin, $filename, \$content);
+               if ($arcname ne '') {	# tnef extraction and conversion successed
+                  $filename=$arcname;
+                  $contenttype=ow::tool::ext2contenttype($filename);
+                  $content=${$r_arcdata};
+               }
+            }
+         }
+
+         if ($contenttype =~ m#^text/html#i ) {			# try to rendering html
             my $escapedfolder = ow::tool::escapeURL($folder);
             my $escapedmessageid = ow::tool::escapeURL($messageid);
             $content = ow::htmlrender::html4nobase($content);
@@ -223,9 +225,7 @@ sub getattachment {
 #            $content = ow::htmlrender::html4mailto($content, "$config{'ow_cgiurl'}/openwebmail-send.pl", "action=composemessage&amp;sort=$sort&amp;keyword=$escapedkeyword&amp;searchtype=$searchtype&amp;folder=$escapedfolder&amp;page=$page&amp;sessionid=$thissession&amp;composetype=sendto");
          }
 
-         my $contenttype = ${$r_attachment}{'content-type'};
-         my $filename = ${$r_attachment}{filename}; $filename=~s/\s$//;
-
+         ($filename)=iconv($charset, $prefs{'charset'}, $filename);
          # remove char disallowed in some fs
          if ($prefs{'charset'} eq 'big5' || $prefs{'charset'} eq 'gb2312') {
             $filename = ow::tool::zh_dospath2fname($filename, '_');	# dos path
@@ -236,28 +236,24 @@ sub getattachment {
          $filename =~ s|^.*:||;	# mac path and dos drive
          $filename=safedlname($filename);
 
-         # we change the filename of an attachment
-         # from *.exe, *.com *.bat, *.pif, *.lnk, *.scr to *.file
-         # if its contenttype is not application/octet-stream
-         # to avoid this attachment is referenced by html and executed directly ie
+         # adjust att fname and contenttype
          if ( $filename =~ /\.(?:exe|com|bat|pif|lnk|scr)$/i &&
               $contenttype !~ /application\/octet\-stream/i &&
               $contenttype !~ /application\/x\-msdownload/i ) {
+            # change attname from *.exe, *.com *.bat, *.pif, *.lnk, *.scr to *.file
+            # if its contenttype is not application/octet-stream,
+            # so this attachment won't be referenced by html and executed directly ie
             $filename="$filename.file";
-         }
-
-         if ( $contenttype =~ /application\/octet\-stream/i &&
-              $filename =~ /\.(jpg|jpeg|gif|png|bmp)$/i ) {
-            # change contenttype of image to make it directly displayed by browser
-            $contenttype="image/".lc($1);
+         } elsif ($filename =~ /\.(?:doc|dot)$/i &&
+             $wordpreview && msword2html(\$content)) {
+            # in wordpreview mode?
+            $contenttype="text/html";
          } elsif ($contenttype =~ /^message\//i) {
             # set message contenttype to text/plain for easy view
             $contenttype = "text/plain";
-         }
-
-         if ($wordpreview && $filename =~ /\.(?:doc|dot)$/i &&	# in wordpreview mode?
-             word2simplehtml(\$content)) {	
-             $contenttype="text/html";
+         } elsif ($contenttype =~ /application\/octet\-stream/i) {
+            # guess a better contenttype so attachment can be better displayed by browser
+            $contenttype=ow::tool::ext2contenttype($filename);
          }
 
          my $length=length($content);
@@ -333,7 +329,7 @@ sub getattfile {
 
    my (%att, $attheader, $attcontent);
    open(ATTFILE, "$config{'ow_sessionsdir'}/$attfile") or
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $config{'ow_sessionsdir'}/$attfile! ($!)");
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_read'} $config{'ow_sessionsdir'}/$attfile! ($!)");
    local $/="\n\n"; $attheader=<ATTFILE>;	# read until 1st blank line
    undef $/; $attcontent=<ATTFILE>;		# read until file end
    close(ATTFILE);
@@ -352,7 +348,7 @@ sub getattfile {
    }
 
    if ($wordpreview && $att{filename} =~ /\.(?:doc|dot)$/i &&	# in wordpreview mode?
-       word2simplehtml(\$attcontent)) {
+       msword2html(\$attcontent)) {
        $attheader=~s!$att{'content-type'}!text/html!;
        $att{'content-type'}="text/html";
    }
@@ -382,23 +378,29 @@ sub savefile2webdisk {
       }
    }
 
+   # webdisksel is from webdisk.pl and it's originally in $prefs{charset}
+   # so we have to convert it into fscharset before do vpath calculation
+   $webdisksel=u2f($webdisksel);
+
    my $webdiskrootdir=ow::tool::untaint($homedir.absolute_vpath("/", $config{'webdisk_rootpath'}));
    my $vpath=absolute_vpath('/', $webdisksel);
+   my $vpathstr=(iconv($prefs{'fscharset'}, $prefs{'charset'}, $vpath))[0];
    my $err=verify_vpath($webdiskrootdir, $vpath);
-   openwebmailerror(__FILE__, __LINE__, $err) if ($err);
+   openwebmailerror(__FILE__, __LINE__, "$lang_err{'access_denied'} ($vpathstr: $err)") if ($err);
 
    if (-d "$webdiskrootdir/$vpath") {			# use choose a dirname, save att with its original name
       $vpath=absolute_vpath($vpath, $filename);
+      $vpathstr=(iconv($prefs{'fscharset'}, $prefs{'charset'}, $vpath))[0];
       $err=verify_vpath($webdiskrootdir, $vpath);
-      openwebmailerror(__FILE__, __LINE__, $err) if ($err);
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'access_denied'} ($vpathstr: $err)") if ($err);
    }
    $vpath=ow::tool::untaint($vpath);
 
    if (!open(F, ">$webdiskrootdir/$vpath") ) {
-      autoclosewindow($lang_text{'savefile'}, "$lang_text{'savefile'} $lang_text{'failed'} ($vpath: $!)");
+      autoclosewindow($lang_text{'savefile'}, "$lang_text{'savefile'} $lang_text{'failed'} ($vpathstr: $!)");
    }
-   ow::filelock::lock("$webdiskrootdir/$vpath", LOCK_EX|LOCK_NB) or
-      autoclosewindow($lang_text{'savefile'}, "$lang_err{'couldnt_lock'} $webdiskrootdir/$vpath!");
+   ow::filelock::lock("$webdiskrootdir/$vpath", LOCK_EX) or
+      autoclosewindow($lang_text{'savefile'}, "$lang_err{'couldnt_writelock'} $vpathstr!");
    print F ${$r_content};
    close(F);
    chmod(0644, "$webdiskrootdir/$vpath");
@@ -407,20 +409,20 @@ sub savefile2webdisk {
    writelog("save attachment - $vpath");
    writehistory("save attachment - $vpath");
 
-   autoclosewindow($lang_text{'savefile'}, "$lang_text{'savefile'} $lang_text{'succeeded'} ($vpath)");
+   autoclosewindow($lang_text{'savefile'}, "$lang_text{'savefile'} $lang_text{'succeeded'} ($vpathstr)");
 }
 ########## END SAVEFILE2WEBDISK ##################################
 
-########## WORD2SIMPLEHTML #######################################
-sub word2simplehtml {
+########## MSWORD2HTML ###########################################
+sub msword2html {
    my $r_content=$_[0];
    my $antiwordbin=ow::tool::findbin('antiword');
    return 0 if ($antiwordbin eq '');
 
-   my $tmpfile=ow::tool::untaint("/tmp/.tmpfile.".time());
+   my $tmpfile=ow::tool::tmpname('msword2html.tmpfile');
    my $err=0;
-   open(F, ">$tmpfile") or return 0; 
-   print F ${$r_content} or $err++; 
+   open(F, ">$tmpfile") or return 0;
+   print F ${$r_content} or $err++;
    close(F);
    if ($err) {
       unlink($tmpfile);
@@ -431,7 +433,7 @@ sub word2simplehtml {
    return 0 if ($exit||$sig);
 
    my $charset=$prefs{'charset'};
-   if (is_convertable('utf-8', $prefs{'charset'}) ) {
+   if (is_convertible('utf-8', $prefs{'charset'}) ) {
       ($stdout)=iconv('utf-8', $prefs{'charset'}, $stdout);
    } else {
       $charset='utf-8';
@@ -441,4 +443,4 @@ sub word2simplehtml {
                  qq|<body><pre>\n$stdout\n</pre></body></html>\n|;
    return 1;
 }
-########## WORD2SIMPLEHTML #######################################
+########## MSWORD2HTML ###########################################
