@@ -5,7 +5,6 @@ use strict;
 use Fcntl qw(:DEFAULT :flock);
 
 use vars qw($_OFFSET $_FROM $_TO $_DATE $_SUBJECT $_CONTENT_TYPE $_STATUS $_SIZE $_REFERENCES $_CHARSET);	# defined in maildb.pl
-use vars qw(%is_defaultfolder);
 use vars qw(%config);
 
 ########## CUTFOLDERMAILS/CUTDIRFILES ############################
@@ -18,7 +17,7 @@ sub cutfoldermails {
    foreach my $f (@folders) {
       ($folderfile{$f},$folderdb{$f})=get_folderpath_folderdb($user, $f);
       my $foldersize=(-s "$folderfile{$f}");
-      if (!$is_defaultfolder{$f}) {
+      if (!is_defaultfolder($f)) {
          push (@userfolders, $f);
          $user_foldersize+=$foldersize;
       }
@@ -33,17 +32,23 @@ sub cutfoldermails {
    }
 
    # empty folders
-   foreach my $f ('mail-trash', 'saved-drafts') {
+   my @f;
+   push(@f, 'virus-mail') if ($config{'has_virusfolder_by_default'});
+   push(@f, 'spam-mail') if ($config{'has_spamfolder_by_default'});
+   push(@f, 'mail-trash');
+   push(@f, 'saved-drafts') if ($config{'enable_savedraft'});
+
+   foreach my $f (@f) {
       next if ( (-s "$folderfile{$f}")==0 );
 
       my $sizereduced = (-s "$folderfile{$f}");
 
-      if (!ow::filelock::lock($folderfile{$f}, LOCK_EX|LOCK_NB)) {
+      if (!ow::filelock::lock($folderfile{$f}, LOCK_EX)) {
          writelog("emptyfolder error - Couldn't get write lock on $folderfile{$f}");
          writehistory("emptyfolder error - Couldn't get write lock on $folderfile{$f}");
          next;
       }
-      my $ret=emptyfolder($folderfile{$f}, $folderdb{$f});
+      my $ret=empty_folder($folderfile{$f}, $folderdb{$f});
       ow::filelock::lock($folderfile{$f}, LOCK_UN);
       if ($ret<0) {
          writelog("emptyfolder error - folder $f ret=$ret");
@@ -59,7 +64,10 @@ sub cutfoldermails {
    }
 
    # cut folders
-   my @folders_tocut=('sent-mail', 'saved-messages');
+   my @folders_tocut;
+   push(@folders_tocut, 'sent-mail') if ($config{'enable_backupsent'});
+   push(@folders_tocut, 'saved-messages');
+
    # put @userfolders to cutlist if it occupies more than 33%
    if ($user_foldersize > $total_foldersize*0.33) {
       push (@folders_tocut, sort(@userfolders));
@@ -87,9 +95,9 @@ sub cutfoldermails {
          my $sizereduced = (-s "$folderfile{$f}");
          my $ret;
          if ($f eq 'sent-mail') {
-            $ret=cutfolder($folderfile{$f}, $folderdb{$f}, $cutpercent+0.1);
+            $ret=_cutfoldermail($folderfile{$f}, $folderdb{$f}, $cutpercent+0.1);
          } else {
-            $ret=cutfolder($folderfile{$f}, $folderdb{$f}, $cutpercent);
+            $ret=_cutfoldermail($folderfile{$f}, $folderdb{$f}, $cutpercent);
          }
          if ($ret<0) {
             writelog("cutfoldermails error - folder $f ret=$ret");
@@ -110,25 +118,29 @@ sub cutfoldermails {
    writehistory("cutfoldermails error - still $sizetocut bytes to cut");
    return ($_[0]-$sizetocut);	# return cutsize
 }
-
-sub cutfolder {				# reduce folder size by $cutpercent
+sub _cutfoldermail {	# reduce folder size by $cutpercent
    my ($folderfile, $folderdb, $cutpercent) = @_;
    my (@delids, $cutsize, %FDB);
 
    ow::filelock::lock($folderfile, LOCK_SH|LOCK_NB) or return -1;
 
-   return -2 if (update_folderindex($folderfile, $folderdb)<0);
-
-   my ($totalsize, $new, $r_messageids)=get_info_messageids_sorted_by_date($folderdb, 0);
-
-   ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or return -3;
+   if (update_folderindex($folderfile, $folderdb)<0) {
+      ow::filelock::lock($folderfile, LOCK_UN);
+      return -2;
+   }
+   my $r_messageids=get_messageids_sorted_by_date($folderdb, 0);
+   if (!ow::dbm::open(\%FDB, $folderdb, LOCK_SH)) {
+      ow::filelock::lock($folderfile, LOCK_UN);
+      return -3;
+   }
+   my $totalsize=(stat($folderfile))[7];
    foreach my $id  (reverse @{$r_messageids}) {
       push(@delids, $id);
       $cutsize += (string2msgattr($FDB{$id}))[$_SIZE];
-      last if ($cutsize > $totalsize*$cutpercent);
+      last if ($cutsize+$FDB{'ZAPSIZE'} > $totalsize*$cutpercent);
    }
    ow::dbm::close(\%FDB, $folderdb);
-   my $counted=operate_message_with_ids("delete", \@delids, $folderfile, $folderdb);
+   my $counted=(operate_message_with_ids("delete", \@delids, $folderfile, $folderdb))[0];
    $counted=folder_zapmessages($folderfile, $folderdb);
 
    ow::filelock::lock($folderfile, LOCK_UN);
@@ -142,7 +154,7 @@ sub cutdirfiles {
    return -1 if (!opendir(D, $dir));
    my @files=readdir(D);
    closedir(D);
-   
+
    my (%ftype, %fdate, %fsize);
 
    foreach my $fname (@files) {
@@ -167,7 +179,7 @@ sub cutdirfiles {
    my @sortedlist= sort { $fdate{$a}<=>$fdate{$b} } keys(%ftype);
    foreach my $fname (@sortedlist) {
       if ($ftype{$fname} eq 'f') {
-         if (unlink("$dir/$fname")) {
+         if (unlink ow::tool::untaint("$dir/$fname")) {
             $sizetocut-=$fsize{$fname};
             writelog("cutdirfiles - file $dir/$fname has been removed");
             writehistory("cutdirfiles - file $dir/$fname has been removed");
@@ -177,11 +189,12 @@ sub cutdirfiles {
          my $sizecut=cutdirfiles($sizetocut, "$dir/$fname");
          if ($sizecut>0) {
             $sizetocut-=$sizecut;
-            if (rmdir("$dir/$fname")) {
+            if (rmdir ow::tool::untaint("$dir/$fname")) {
                writelog("cutdir - dir $dir/$fname has been removed");
                writehistory("cutdir - dir $dir/$fname has been removed");
             } else {
-               utime($now, $fdate{$fname}, "$dir/$fname");	# set modify time back
+               # restore dir modify time
+               utime($now, ow::tool::untaint($fdate{$fname}), ow::tool::untaint("$dir/$fname"));
             }
             return ($_[0]-$sizetocut) if ($sizetocut<=0);	# return cutsize
          }
