@@ -19,14 +19,18 @@ sub getinfomessageids {
 
    # do new indexing in background if folder > 10 M && empty db
    if (!ow::dbm::exist($folderdb) && (-s $folderfile) >= 10485760) {
-      local $|=1; # flush all output
-      local $SIG{CHLD} = sub { wait; $_index_complete=1 if ($?==0) };	# handle zombie
       local $_index_complete=0;
+      local $SIG{CHLD} = sub { wait; $_index_complete=1 if ($?==0) };	# signaled when indexing completes
+      local $|=1; # flush all output
       if ( fork() == 0 ) {		# child
          close(STDIN); close(STDOUT); close(STDERR);
+         writelog("debug - update folderindex process forked - " .__FILE__.":". __LINE__) if ($config{'debug_fork'});
+
          ow::filelock::lock($folderfile, LOCK_SH|LOCK_NB) or openwebmail_exit(1);
          update_folderindex($folderfile, $folderdb);
          ow::filelock::lock($folderfile, LOCK_UN);
+
+         writelog("debug - update folderindex process terminated - " .__FILE__.":". __LINE__) if ($config{'debug_fork'});
          openwebmail_exit(0);
       }
 
@@ -36,14 +40,14 @@ sub getinfomessageids {
       }
 
       if ($_index_complete==0) {
-         openwebmailerror(__FILE__, __LINE__, "$folderfile $lang_err{'under_indexing'}");
+         openwebmailerror(__FILE__, __LINE__, f2u($folderfile)." $lang_err{'under_indexing'}");
       }
    } else {	# do indexing directly if small folder
       ow::filelock::lock($folderfile, LOCK_SH|LOCK_NB) or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $folderfile!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_readlock'} ".f2u($folderfile)."!");
       if (update_folderindex($folderfile, $folderdb)<0) {
          ow::filelock::lock($folderfile, LOCK_UN);
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} db $folderdb");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_updatedb'} db ".f2u($folderdb));
       }
       ow::filelock::lock($folderfile, LOCK_UN);
    }
@@ -58,21 +62,34 @@ sub getinfomessageids {
       $sort='recipient' if ($sort eq 'sender');
    }
 
+   my ($totalsize, $new, $r_messageids, $r_messagedepths);
+
    if ( $keyword ne '' ) {
       my $folderhandle=do { local *FH };
-      my ($totalsize, $new, $r_haskeyword, $r_messageids, $r_messagedepths);
+      my %FDB;
+      my $r_haskeyword;
       my @messageids=();
       my @messagedepths=();
 
-      ($totalsize, $new, $r_messageids, $r_messagedepths)=get_info_messageids_sorted($folderdb, $sort, "$folderdb.cache", $prefs{'hideinternal'});
+      ($r_messageids, $r_messagedepths)=get_messageids_sorted($folderdb, $sort, "$folderdb.cache", $prefs{'hideinternal'});
 
       ow::filelock::lock($folderfile, LOCK_SH|LOCK_NB) or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_locksh'} $folderfile!");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_readlock'} ".f2u($folderfile)."!");
+
       open($folderhandle, $folderfile);
-      ($totalsize, $new, $r_haskeyword)=search_info_messages_for_keyword(
-					$keyword, $prefs{'charset'}, $searchtype, $folderdb, $folderhandle,
-					dotpath('search.cache'), $prefs{'hideinternal'}, $prefs{'regexmatch'});
+      $r_haskeyword=search_info_messages_for_keyword(
+			$keyword, $prefs{'charset'}, $searchtype, $folderdb, $folderhandle,
+			dotpath('search.cache'), $prefs{'hideinternal'}, $prefs{'regexmatch'});
       close($folderhandle);
+
+      ow::dbm::open(\%FDB, $folderdb, LOCK_SH);
+      foreach my $messageid (keys %{$r_haskeyword}) {
+         my @attr=string2msgattr($FDB{$messageid});
+         $new++ if ($attr[$_STATUS]!~/R/i);
+         $totalsize+=$attr[$_SIZE];
+      }
+      ow::dbm::close(\%FDB, $folderdb);
+
       ow::filelock::lock($folderfile, LOCK_UN);
 
       for (my $i=0; $i<@{$r_messageids}; $i++) {
@@ -85,7 +102,18 @@ sub getinfomessageids {
       return($totalsize, $new, \@messageids, \@messagedepths);
 
    } else { # return: $totalsize, $new, $r_messageids for whole folder
-      return(get_info_messageids_sorted($folderdb, $sort, "$folderdb.cache", $prefs{'hideinternal'}))
+      my %FDB;
+
+      ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or
+         return($totalsize, $new, $r_messageids, $r_messagedepths);
+      $new=$FDB{'NEWMESSAGES'};
+      $totalsize=(stat($folderfile))[7];
+      $totalsize=$totalsize-$FDB{'INTERNALSIZE'}-$FDB{'ZAPSIZE'};
+      ow::dbm::close(\%FDB, $folderdb);
+
+      ($r_messageids, $r_messagedepths)=get_messageids_sorted($folderdb, $sort, "$folderdb.cache", $prefs{'hideinternal'});
+
+      return($totalsize, $new, $r_messageids, $r_messagedepths);
    }
 }
 
@@ -93,48 +121,56 @@ sub getinfomessageids {
 # prefs_charset: the charset of the keyword
 sub search_info_messages_for_keyword {
    my ($keyword, $prefs_charset, $searchtype, $folderdb, $folderhandle, $cachefile, $ignore_internal, $regexmatch)=@_;
-   my ($cache_metainfo, $cache_folderdb, $cache_keyword, $cache_searchtype, $cache_ignore_internal);
+   my ($cache_lstmtime, $cache_folderdb, $cache_keyword, $cache_searchtype, $cache_ignore_internal);
    my (%FDB, @messageids, $messageid);
-   my ($totalsize, $new)=(0,0);
    my %found=();
 
    ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or
-      return($totalsize, $new, \%found);
-   my $metainfo=$FDB{'METAINFO'};
+      return(\%found);
+   my $lstmtime=$FDB{'LSTMTIME'};
    ow::dbm::close(\%FDB, $folderdb);
 
    ow::filelock::lock($cachefile, LOCK_EX) or
-      return($totalsize, $new, \%found);
+      return(\%found);
 
    if ( -e $cachefile ) {
       open(CACHE, $cachefile);
-      foreach ($cache_metainfo, $cache_folderdb, $cache_keyword, $cache_searchtype, $cache_ignore_internal) {
+      foreach ($cache_lstmtime, $cache_folderdb, $cache_keyword, $cache_searchtype, $cache_ignore_internal) {
          $_=<CACHE>; chomp;
       }
       close(CACHE);
    }
 
-   if ( $cache_metainfo ne $metainfo || $cache_folderdb ne $folderdb ||
+   if ( $cache_lstmtime ne $lstmtime || $cache_folderdb ne $folderdb ||
         $cache_keyword ne $keyword || $cache_searchtype ne $searchtype ||
         $cache_ignore_internal ne $ignore_internal ) {
       $cachefile=ow::tool::untaint($cachefile);
       @messageids=get_messageids_sorted_by_offset($folderdb, $folderhandle);
 
-      ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or return($totalsize, $new, \%found);
+      ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or
+         return(\%found);
 
       # check if keyword a valid regex
       $regexmatch = $regexmatch && ow::tool::is_regex($keyword);
 
       foreach $messageid (@messageids) {
-         my (@attr, @references, $block, $header, $body, $r_attachments) ;
+         my (@attr, $date, $block, $header, $body, $r_attachments);
          @attr=string2msgattr($FDB{$messageid});
+         next if ($attr[$_STATUS]=~/Z/i);
          next if ($ignore_internal && is_internal_subject($attr[$_SUBJECT]));
-         @references=split(/\s+/, $attr[$_REFERENCES]);
 
-         my $is_conv=is_convertable($attr[$_CHARSET], $prefs_charset);
-         if ($is_conv) {
-            ($attr[$_FROM], $attr[$_TO], $attr[$_SUBJECT])=
-               iconv($attr[$_CHARSET], $prefs_charset, $attr[$_FROM], $attr[$_TO], $attr[$_SUBJECT]);
+         my $msgcharset=$attr[$_CHARSET];
+         if ($msgcharset eq '' && $prefs_charset eq 'utf-8') {
+            # assume msg is from sender using same language as the recipient's browser
+            $msgcharset=$ow::lang::languagecharsets{ow::lang::guess_browser_language()};
+         }
+         ($attr[$_FROM], $attr[$_TO], $attr[$_SUBJECT])=
+               iconv($msgcharset, $prefs_charset, $attr[$_FROM], $attr[$_TO], $attr[$_SUBJECT]);
+
+         if ($searchtype eq 'all' || $searchtype eq 'date') {
+            $date=ow::datetime::dateserial2str($attr[$_DATE],
+                                               $prefs{'timeoffset'}, $prefs{'daylightsaving'},
+                                               $prefs{'dateformat'}, $prefs{'hourformat'});
          }
 
          # check subject, from, to, date
@@ -152,23 +188,22 @@ sub search_info_messages_for_keyword {
                  $attr[$_TO]=~/\Q$keyword\E/i) )  ||
               ( ($searchtype eq 'all' ||
                  $searchtype eq 'date') &&
-                (($regexmatch && $attr[$_DATE]=~/$keyword/i) ||
-                 $attr[$_DATE]=~/\Q$keyword\E/i) )
+                (($regexmatch && $date=~/$keyword/i) ||
+                 $date=~/\Q$keyword\E/i) )
             ) {
             $found{$messageid}=1;
          }
          # try to find msgs in same thread with references if seaching subject
          if ($searchtype eq 'subject') {
+            my @references=split(/\s+/, $attr[$_REFERENCES]);
             foreach my $refid (@references) {
                # if a msg is already in %found, then we put all msgs it references in %found
-               $found{$refid}=1 if ($found{$messageid} && defined($FDB{$refid}));
+               $found{$refid}=1 if ($found{$messageid} && defined $FDB{$refid});
                # if a msg references any member in %found, thn we put this msg in %found
                $found{$messageid}=1 if ($found{$refid});
             }
          }
          if ($found{$messageid}) {
-            $new++ if ($attr[$_STATUS]!~/r/i);
-            $totalsize+=$attr[$_SIZE];
             next;
          }
 
@@ -181,14 +216,12 @@ sub search_info_messages_for_keyword {
                $header.=$_;
                last if ($_ eq "\n");
             }
-            $header = ow::mime::decode_mimewords($header);
+            $header = decode_mimewords_iconv($header, $msgcharset);
             $header=~s/\n / /g;	# handle folding roughly
-            ($header)=iconv($attr[$_CHARSET], $prefs_charset, $header) if ($is_conv);
+            ($header)=iconv($msgcharset, $prefs_charset, $header);
 
             if ( ($regexmatch && $header =~ /$keyword/im) ||
                  $header =~ /\Q$keyword\E/im ) {
-               $new++ if ($attr[$_STATUS]!~/r/i);
-               $totalsize+=$attr[$_SIZE];
                $found{$messageid}=1;
                next;
             }
@@ -213,11 +246,9 @@ sub search_info_messages_for_keyword {
                } elsif ($header =~ /content-transfer-encoding:\s+x-uuencode/i) {
                   $body = ow::mime::uudecode($body);
                }
-               ($body)=iconv($attr[$_CHARSET], $prefs_charset, $body) if ($is_conv);
+               ($body)=iconv($msgcharset, $prefs_charset, $body);
                if ( ($regexmatch && $body =~ /$keyword/im) ||
                     $body =~ /\Q$keyword\E/im ) {
-                  $new++ if ($attr[$_STATUS]!~/r/i);
-                  $totalsize+=$attr[$_SIZE];
                   $found{$messageid}=1;
                   next;
                }
@@ -236,15 +267,11 @@ sub search_info_messages_for_keyword {
                   } else {
                      $content=${${$r_attachment}{r_content}};
                   }
-                  my $attcharset=${$r_attachment}{charset}||$attr[$_CHARSET];
-                  if (is_convertable($attcharset, $prefs_charset)) {
-                     ($content)=iconv($attcharset, $prefs_charset, $content);
-                  }
+                  my $attcharset=${$r_attachment}{charset}||$msgcharset;
+                  ($content)=iconv($attcharset, $prefs_charset, $content);
 
                   if ( ($regexmatch && $content =~ /$keyword/im) ||
                        $content =~ /\Q$keyword\E/im ) {
-                     $new++ if ($attr[$_STATUS]!~/r/i);
-                     $totalsize+=$attr[$_SIZE];
                      $found{$messageid}=1;
                      last;	# leave attachments check in one message
                   }
@@ -255,15 +282,11 @@ sub search_info_messages_for_keyword {
 	 # check attfilename
          if ($searchtype eq 'all' || $searchtype eq 'attfilename') {
             foreach my $r_attachment (@{$r_attachments}) {
-               my $filename=${$r_attachment}{filename};
-               my $attcharset=${$r_attachment}{filenamecharset}||${$r_attachment}{charset}||$attr[$_CHARSET];
-               if (is_convertable($attcharset, $prefs_charset)) {
-                  ($filename)=iconv($attcharset, $prefs_charset, $filename);
-               }
+               my $attcharset=${$r_attachment}{filenamecharset}||${$r_attachment}{charset}||$msgcharset;
+               my ($filename)=iconv($attcharset, $prefs_charset, ${$r_attachment}{filename});
+
                if ( ($regexmatch && $filename =~ /$keyword/im) ||
                     $filename =~ /\Q$keyword\E/im ) {
-                  $new++ if ($attr[$_STATUS]!~/r/i);
-                  $totalsize+=$attr[$_SIZE];
                   $found{$messageid}=1;
                   last;	# leave attachments check in one message
                }
@@ -274,17 +297,15 @@ sub search_info_messages_for_keyword {
       ow::dbm::close(\%FDB, $folderdb);
 
       open(CACHE, ">$cachefile") or logtime("cache write error $!");
-      foreach ($metainfo, $folderdb, $keyword, $searchtype, $ignore_internal) {
+      foreach ($lstmtime, $folderdb, $keyword, $searchtype, $ignore_internal) {
          print CACHE $_, "\n";
       }
-      print CACHE join("\n", $totalsize, $new, keys(%found));
+      print CACHE join("\n", keys(%found));
       close(CACHE);
 
    } else {
       open(CACHE, $cachefile);
       for (0..4) { $_=<CACHE>; }	# skip 5 lines
-      $totalsize=<CACHE>; chomp($totalsize);
-      $new=<CACHE>; chomp($new);
       while (<CACHE>) {
          chomp; $found{$_}=1;
       }
@@ -293,112 +314,188 @@ sub search_info_messages_for_keyword {
 
    ow::filelock::lock($cachefile, LOCK_UN);
 
-   return($totalsize, $new, \%found);
+   return(\%found);
 }
 
 ########## GET_MESSAGEIDS_SORTED_BY_...  #########################
 
-sub get_info_messageids_sorted_by_date {
+use vars qw(%sorttype);
+%sorttype= (
+   'date'          => ['date', 0],
+   'date_rev'      => ['date', 1],
+   'sender'        => ['sender', 0],
+   'sender_rev'    => ['sender', 1],
+   'recipient'     => ['recipient', 0],
+   'recipient_rev' => ['recipient', 1],
+   'size'          => ['size', 0],
+   'size_rev'      => ['size', 1],
+   'subject'       => ['subject', 1],
+   'subject_rev'   => ['subject', 0],
+   'status'        => ['status', 0],
+   'status_rev'    => ['status', 1]
+   );
+
+sub get_messageids_sorted {
+   my ($folderdb, $sort, $cachefile, $ignore_internal)=@_;
+   my ($cache_lstmtime, $cache_folderdb, $cache_sort, $cache_ignore_internal);
+   my %FDB;
+   my $r_messageids;
+   my $r_messagedepths;
+   my @messageids=();
+   my @messagedepths=();
+   my $messageids_size;
+   my $messagedepths_size;
+   my $rev;
+
+   if (defined $sorttype{$sort}) {
+      ($sort, $rev)=@{$sorttype{$sort}};
+   } else {
+      ($sort, $rev)= ('date', 0);
+   }
+
+   ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or
+      return(\@messageids, \@messagedepths);
+   my $lstmtime=$FDB{'LSTMTIME'};
+   ow::dbm::close(\%FDB, $folderdb);
+
+   ow::filelock::lock($cachefile, LOCK_EX) or
+      return(\@messageids, \@messagedepths);
+
+   if ( -e $cachefile ) {
+      open(CACHE, $cachefile);
+      foreach ($cache_lstmtime, $cache_folderdb, $cache_sort, $cache_ignore_internal) {
+         $_=<CACHE>; chomp;
+      }
+      close(CACHE);
+   }
+
+   # LSTMTIME will be upated in case the message list of a folder is changed,
+   # eg: 1. db is rebuild
+   #     2. messages added into or removed from db
+   # But LSTMTIME won't be changed in message_status_update.
+   # so we don't have to reload msglist from db after a message status is changed
+   if ( $cache_lstmtime ne $lstmtime || $cache_folderdb ne $folderdb ||
+        $cache_sort ne $sort || $cache_ignore_internal ne $ignore_internal) {
+      $cachefile=ow::tool::untaint($cachefile);
+      open(CACHE, ">$cachefile");
+      print CACHE $lstmtime, "\n", $folderdb, "\n", $sort, "\n", $ignore_internal, "\n";
+      if ( $sort eq 'date' ) {
+         ($r_messageids)=get_messageids_sorted_by_date($folderdb, $ignore_internal);
+      } elsif ( $sort eq 'sender' ) {
+         ($r_messageids)=get_messageids_sorted_by_from($folderdb, $ignore_internal);
+      } elsif ( $sort eq 'recipient' ) {
+         ($r_messageids)=get_messageids_sorted_by_to($folderdb, $ignore_internal);
+      } elsif ( $sort eq 'size' ) {
+         ($r_messageids)=get_messageids_sorted_by_size($folderdb, $ignore_internal);
+      } elsif ( $sort eq 'subject' ) {
+         ($r_messageids, $r_messagedepths)=get_messageids_sorted_by_subject($folderdb, $ignore_internal);
+      } elsif ( $sort eq 'status' ) {
+         ($r_messageids)=get_messageids_sorted_by_status($folderdb, $ignore_internal);
+      }
+
+      $messageids_size = @{$r_messageids};
+
+      @messagedepths=@{$r_messagedepths} if $r_messagedepths;
+      $messagedepths_size = @messagedepths;
+
+      print CACHE join("\n", $messageids_size, $messagedepths_size, @{$r_messageids}, @messagedepths);
+      close(CACHE);
+      if ($rev) {
+         @messageids=reverse @{$r_messageids};
+         @messagedepths=reverse @{$r_messagedepths} if $r_messagedepths;
+      } else {
+         @messageids=@{$r_messageids};
+         @messagedepths=@{$r_messagedepths} if $r_messagedepths;
+      }
+
+   } else {
+      open(CACHE, $cachefile);
+      for (0..3) { $_=<CACHE>; }	# skip $lstmtime, $folderdb, $sort, $ignore_internal
+      foreach ($messageids_size, $messagedepths_size) {
+         $_=<CACHE>; chomp;
+      }
+      my $i = 0;
+      while (<CACHE>) {
+         chomp;
+         if ($rev) {
+            if ($i < $messageids_size) { unshift (@messageids, $_); }
+            else { unshift (@messagedepths, $_); }
+         } else {
+            if ($i < $messageids_size) { push (@messageids, $_); }
+            else { push (@messagedepths, $_); }
+         }
+	 $i++;
+      }
+      close(CACHE);
+   }
+   ow::filelock::lock($cachefile, LOCK_UN);
+
+   return(\@messageids, \@messagedepths);
+}
+
+
+sub get_messageids_sorted_by_date {
    my ($folderdb, $ignore_internal)=@_;
 
-   my ($totalsize, $total, $new, $r_msgid2attrs)
-      =get_info_msgid2attrs($folderdb, $ignore_internal, $_DATE);
+   my ($total, $r_msgid2attrs)=get_msgid2attrs($folderdb, $ignore_internal, $_DATE);
    my @messageids= sort {
                         ${${$r_msgid2attrs}{$b}}[0]<=>${${$r_msgid2attrs}{$a}}[0];
                         } keys %{$r_msgid2attrs};
-
-   return($totalsize, $new, \@messageids);
+   return(\@messageids);
 }
 
-sub get_info_messageids_sorted_by_from {
+sub get_messageids_sorted_by_from {
    my ($folderdb, $ignore_internal)=@_;
 
-   my ($totalsize, $total, $new, $r_msgid2attrs)
-      =get_info_msgid2attrs($folderdb, $ignore_internal, $_DATE, $_FROM);
+   my ($total, $r_msgid2attrs)=get_msgid2attrs($folderdb, $ignore_internal, $_DATE, $_FROM);
+
+   my %msgfromname=();
+   foreach my $id (keys %{$r_msgid2attrs}) {
+      $msgfromname{$id}= lc ( (ow::tool::email2nameaddr(${${$r_msgid2attrs}{$id}}[1]))[0] );
+   }
    my @messageids= sort {
+                        $msgfromname{$a} cmp $msgfromname{$b} or
                         ${${$r_msgid2attrs}{$b}}[0]<=>${${$r_msgid2attrs}{$a}}[0];
-                        } keys %{$r_msgid2attrs};
-
-   # try to group message of same 'from'
-   my %groupdate=();
-   my %groupmembers=();
-   foreach my $key (@messageids) {
-      my $from=${${$r_msgid2attrs}{$key}}[1];
-      if ( !defined($groupdate{$from}) ) {
-         my @members=($key);
-         $groupmembers{$from}=\@members;
-         $groupdate{$from}=${${$r_msgid2attrs}{$key}}[0];
-      } else {
-         push(@{$groupmembers{$from}}, $key);
-      }
-   }
-   @messageids=();
-
-   # sort group by groupdate
-   my @froms=sort {$groupdate{$b} <=> $groupdate{$a}} keys(%groupdate);
-   foreach my $from (@froms) {
-      push(@messageids, @{$groupmembers{$from}});
-   }
-
-   return($totalsize, $new, \@messageids);
+                        } keys %msgfromname;
+   return(\@messageids);
 }
 
-sub get_info_messageids_sorted_by_to {
+sub get_messageids_sorted_by_to {
    my ($folderdb, $ignore_internal)=@_;
 
-   my ($totalsize, $total, $new, $r_msgid2attrs)
-      =get_info_msgid2attrs($folderdb, $ignore_internal, $_DATE, $_TO);
+   my ($total, $r_msgid2attrs)=get_msgid2attrs($folderdb, $ignore_internal, $_DATE, $_TO);
+
+   my %msgtoname=();
+   foreach my $id (keys %{$r_msgid2attrs}) {
+      my @tos=ow::tool::str2list(${${$r_msgid2attrs}{$id}}[1]);
+      $msgtoname{$id}= lc( (ow::tool::email2nameaddr($tos[0]))[0] );
+   }
    my @messageids= sort {
-                        ${${$r_msgid2attrs}{$b}}[0]<=>${${$r_msgid2attrs}{$a}}[0]
-                        } keys(%{$r_msgid2attrs});
-
-   # try to group message of same 'to'
-   my %groupdate=();
-   my %groupmembers=();
-   foreach my $key (@messageids) {
-      my $to=${${$r_msgid2attrs}{$key}}[1];
-      if ( !defined($groupdate{$to}) ) {
-         my @members=($key);
-         $groupmembers{$to}=\@members;
-         $groupdate{$to}=${${$r_msgid2attrs}{$key}}[0];
-      } else {
-         push(@{$groupmembers{$to}}, $key);
-      }
-   }
-   @messageids=();
-
-   # sort group by groupdate
-   my @froms=sort {$groupdate{$b} <=> $groupdate{$a}} keys %groupdate;
-   foreach my $from (@froms) {
-      push(@messageids, @{$groupmembers{$from}});
-   }
-
-   return($totalsize, $new, \@messageids);
+                        $msgtoname{$a} cmp $msgtoname{$b} or
+                        ${${$r_msgid2attrs}{$b}}[0]<=>${${$r_msgid2attrs}{$a}}[0];
+                        } keys %msgtoname;
+   return(\@messageids);
 }
 
-sub get_info_messageids_sorted_by_size {
+sub get_messageids_sorted_by_size {
    my ($folderdb, $ignore_internal)=@_;
 
-   my ($totalsize, $total, $new, $r_msgid2attrs)
-      =get_info_msgid2attrs($folderdb, $ignore_internal, $_DATE, $_SIZE);
+   my ($total, $r_msgid2attrs)=get_msgid2attrs($folderdb, $ignore_internal, $_DATE, $_SIZE);
    my @messageids= sort {
                         ${${$r_msgid2attrs}{$b}}[1]<=>${${$r_msgid2attrs}{$a}}[1] or
                         ${${$r_msgid2attrs}{$b}}[0]<=>${${$r_msgid2attrs}{$a}}[0]
                         } keys %{$r_msgid2attrs};
-
-   return($totalsize, $new, \@messageids);
+   return(\@messageids);
 }
 
-sub get_info_messageids_sorted_by_status {
+sub get_messageids_sorted_by_status {
    my ($folderdb, $ignore_internal)=@_;
 
-   my ($totalsize, $total, $new, $r_msgid2attrs)
-      =get_info_msgid2attrs($folderdb, $ignore_internal, $_DATE, $_STATUS);
-
+   my ($total, $r_msgid2attrs)=get_msgid2attrs($folderdb, $ignore_internal, $_DATE, $_STATUS);
    my %status;
    foreach my $key (keys %{$r_msgid2attrs}) {
       my $status=${${$r_msgid2attrs}{$key}}[1];
-      if ($status=~/r/i) {
+      if ($status=~/R/i) {
          $status{$key}=0;
       } else {
          $status{$key}=2;
@@ -409,17 +506,15 @@ sub get_info_messageids_sorted_by_status {
                        $status{$b} <=> $status{$a} or
                        ${${$r_msgid2attrs}{$b}}[0]<=>${${$r_msgid2attrs}{$a}}[0]
                        } keys %status;
-
-   return($totalsize, $new, \@messageids);
+   return(\@messageids);
 }
 
 # this routine actually sorts messages by thread,
 # contributed by <james@tiger-marmalade.com"> James Dean Palmer
-sub get_info_messageids_sorted_by_subject {
+sub get_messageids_sorted_by_subject {
    my ($folderdb, $ignore_internal)=@_;
 
-   my ($totalsize, $total, $new, $r_msgid2attrs)
-      =get_info_msgid2attrs($folderdb, $ignore_internal, $_DATE, $_REFERENCES, $_SUBJECT);
+   my ($total, $r_msgid2attrs)=get_msgid2attrs($folderdb, $ignore_internal, $_DATE, $_REFERENCES, $_SUBJECT);
 
    my (%subject, %date);
    foreach my $key (keys %{$r_msgid2attrs}) {
@@ -439,7 +534,7 @@ sub get_info_messageids_sorted_by_subject {
       my @parents = reverse split(/ /, ${${$r_msgid2attrs}{$key}}[1]); # most nearby first
       my $parent = "ROOT.nonexist";	# this should be a string that would never be used as a messageid
       foreach my $id (@parents) {
-         if ( defined($subject{$id}) ) {
+         if (defined $subject{$id}) {
  	    $parent = $id;
 	    last;
          }
@@ -453,7 +548,7 @@ sub get_info_messageids_sorted_by_subject {
    # so we should connect them with the earliest article by the same title.
    @thread_pre_roots = sort {
                             $subject{$a} cmp $subject{$b} or
-                            $date{$a} cmp $date{$b}
+                            $date{$a} <=> $date{$b}
                             } @thread_pre_roots;
    my $previous_id = "";
    foreach my $id (@thread_pre_roots) {
@@ -491,7 +586,7 @@ sub get_info_messageids_sorted_by_subject {
       _recursively_thread ($key, 0,
 		\@message_ids, \@message_depths, \%thread_children, \%date);
    }
-   return($totalsize, $new, \@message_ids, \@message_depths);
+   return(\@message_ids, \@message_depths);
 }
 
 sub _recursively_thread {
@@ -500,7 +595,7 @@ sub _recursively_thread {
 
    push @{$r_message_ids}, $id;
    push @{$r_message_depths}, $depth;
-   if (defined(${$r_thread_children}{$id})) {
+   if (defined ${$r_thread_children}{$id}) {
       my @children = sort { ${$r_date}{$a} <=> ${$r_date}{$b}; } @{${$r_thread_children}{$id}};
       foreach my $thread (@children) {
          _recursively_thread ($thread, $depth+1,
@@ -510,118 +605,6 @@ sub _recursively_thread {
    return;
 }
 
-use vars qw(%sorttype);
-%sorttype= (
-   'date'          => ['date', 0],
-   'date_rev'      => ['date', 1],
-   'sender'        => ['sender', 0],
-   'sender_rev'    => ['sender', 1],
-   'recipient'     => ['recipient', 0],
-   'recipient_rev' => ['recipient', 1],
-   'size'          => ['size', 0],
-   'size_rev'      => ['size', 1],
-   'subject'       => ['subject', 1],
-   'subject_rev'   => ['subject', 0],
-   'status'        => ['status', 0],
-   'status_rev'    => ['status', 1]
-   );
-
-sub get_info_messageids_sorted {
-   my ($folderdb, $sort, $cachefile, $ignore_internal)=@_;
-   my ($cache_metainfo, $cache_folderdb, $cache_sort, $cache_ignore_internal);
-   my %FDB;
-   my ($totalsize, $new)=(0,0);
-   my $r_messageids;
-   my $r_messagedepths;
-   my @messageids=();
-   my @messagedepths=();
-   my $messageids_size;
-   my $messagedepths_size;
-   my $rev;
-
-   if (defined($sorttype{$sort})) {
-      ($sort, $rev)=@{$sorttype{$sort}};
-   } else {
-      ($sort, $rev)= ('date', 0);
-   }
-
-   ow::dbm::open(\%FDB, $folderdb, LOCK_SH) or 
-      return($totalsize, $new, \@messageids, \@messagedepths);
-   my $metainfo=$FDB{'METAINFO'};
-   ow::dbm::close(\%FDB, $folderdb);
-
-   ow::filelock::lock($cachefile, LOCK_EX) or
-      return($totalsize, $new, \@messageids, \@messagedepths);
-
-   if ( -e $cachefile ) {
-      open(CACHE, $cachefile);
-      foreach ($cache_metainfo, $cache_folderdb, $cache_sort, $cache_ignore_internal, $totalsize) {
-         $_=<CACHE>; chomp;
-      }
-      close(CACHE);
-   }
-
-   if ( $cache_metainfo ne $metainfo || $cache_folderdb ne $folderdb ||
-        $cache_sort ne $sort || $cache_ignore_internal ne $ignore_internal ||
-        $totalsize=~/[^\d]/ ) {
-      $cachefile=ow::tool::untaint($cachefile);
-      open(CACHE, ">$cachefile");
-      print CACHE $metainfo, "\n", $folderdb, "\n", $sort, "\n", $ignore_internal, "\n";
-      if ( $sort eq 'date' ) {
-         ($totalsize, $new, $r_messageids)=get_info_messageids_sorted_by_date($folderdb, $ignore_internal);
-      } elsif ( $sort eq 'sender' ) {
-         ($totalsize, $new, $r_messageids)=get_info_messageids_sorted_by_from($folderdb, $ignore_internal);
-      } elsif ( $sort eq 'recipient' ) {
-         ($totalsize, $new, $r_messageids)=get_info_messageids_sorted_by_to($folderdb, $ignore_internal);
-      } elsif ( $sort eq 'size' ) {
-         ($totalsize, $new, $r_messageids)=get_info_messageids_sorted_by_size($folderdb, $ignore_internal);
-      } elsif ( $sort eq 'subject' ) {
-         ($totalsize, $new, $r_messageids, $r_messagedepths)=get_info_messageids_sorted_by_subject($folderdb, $ignore_internal);
-      } elsif ( $sort eq 'status' ) {
-         ($totalsize, $new, $r_messageids)=get_info_messageids_sorted_by_status($folderdb, $ignore_internal);
-      }
-
-      $messageids_size = @{$r_messageids};
-
-      @messagedepths=@{$r_messagedepths} if $r_messagedepths;
-      $messagedepths_size = @messagedepths;
-
-      print CACHE join("\n", $totalsize, $new, $messageids_size, $messagedepths_size, @{$r_messageids}, @messagedepths);
-      close(CACHE);
-      if ($rev) {
-         @messageids=reverse @{$r_messageids};
-         @messagedepths=reverse @{$r_messagedepths} if $r_messagedepths;
-      } else {
-         @messageids=@{$r_messageids};
-         @messagedepths=@{$r_messagedepths} if $r_messagedepths;
-      }
-
-   } else {
-      open(CACHE, $cachefile);
-      for (0..3) { $_=<CACHE>; }	# skip 4 lines
-      foreach ($totalsize, $new, $messageids_size, $messagedepths_size) {
-         $_=<CACHE>; chomp;
-      }
-      my $i = 0;
-      while (<CACHE>) {
-         chomp;
-         if ($rev) {
-            if ($i < $messageids_size) { unshift (@messageids, $_); }
-            else { unshift (@messagedepths, $_); }
-         } else {
-            if ($i < $messageids_size) { push (@messageids, $_); }
-            else { push (@messagedepths, $_); }
-         }
-	 $i++;
-      }
-      close(CACHE);
-   }
-
-   ow::filelock::lock($cachefile, LOCK_UN);
-
-   return($totalsize, $new, \@messageids, \@messagedepths);
-}
-
 ########## END GET_MESSAGEIDS_SORTED_BY_...  #####################
 
 1;
