@@ -5,10 +5,11 @@
 #
 use strict;
 use Text::Iconv;
+require "modules/mime.pl";
 require "shares/iconv-chinese.pl";
 require "shares/iconv-japan.pl";
 
-use vars qw(%charset_convlist %charset_localname %localname_cache);
+use vars qw(%charset_convlist %charset_equiv %charset_localname);
 
 # mapping www charset to all possible iconv charset on various platform
 %charset_localname=
@@ -17,7 +18,6 @@ use vars qw(%charset_convlist %charset_localname %localname_cache);
    'euc-jp'        => [ 'EUC-JP', 'EUC', 'eucJP' ],
    'euc-kr'        => [ 'EUC-KR', 'EUCKR' ],
    'gb2312'        => [ 'GB2312', 'gb2312' ],
-   'gbk'           => [ 'GBK', 'gbk' ],
    'iso-2022-jp'   => [ 'ISO-2022-JP', 'JIS' ],
    'iso-2022-kr'   => [ 'ISO-2022-KR' ],
    'iso-8859-1'    => [ 'ISO-8859-1', '8859-1', 'ISO8859-1', 'ISO_8859-1' ],
@@ -31,7 +31,6 @@ use vars qw(%charset_convlist %charset_localname %localname_cache);
    'koi8-r'        => [ 'KOI8-R' ],
    'koi8-u'        => [ 'KOI8-U' ],
    'ksc5601'       => [ 'KSC5601' ],
-   'ks_c_5601-1987'=> [ 'KSC5601' ],
    'shift_jis'     => [ 'SJIS', 'SHIFT_JIS', 'SHIFT-JIS' ],
    'tis-620'       => [ 'TIS-620', 'TIS620' ],
    'utf-8'         => [ 'UTF-8', 'UTF8' ],
@@ -45,18 +44,28 @@ use vars qw(%charset_convlist %charset_localname %localname_cache);
    'windows-1257'  => [ 'WINDOWS-1257', 'CP1257' ],
    );
 
-# convertable list of WWW charset, the definition is:
+# map old/unofficial charset name to official charset name
+%charset_equiv=
+   (
+   'big-5'          => 'big5',
+   'chinesebig5'    => 'big5',
+   'gbk'            => 'gb2312',
+   'iso-8859'       => 'iso-8859-1',
+   'us-ascii'       => 'iso-8859-1',
+   'ks_c_5601-1987' => 'ksc5601',
+   'utf8'           => 'utf-8'
+   );
+
+# convertible list of WWW charset, the definition is:
 # charset in the left can be converted from the charsets in right list
 %charset_convlist=
    (
-   'big5'          => [ 'utf-8', 'gb2312', 'gbk' ],
+   'big5'          => [ 'utf-8', 'gb2312'],
    'euc-jp'        => [ 'utf-8', 'iso-2022-jp', 'shift_jis' ],
-   'euc-kr'        => [ 'utf-8', 'ks_c_5601-1987', 'ksc5601', 'iso-2022-kr' ],
-   'iso-2022-kr'   => [ 'utf-8', 'ks_c_5601-1987', 'ksc5601', 'euc-kr' ],
-   'ks_c_5601-1987'=> [ 'utf-8', 'euc-kr', 'iso-2022-kr' ],
+   'euc-kr'        => [ 'utf-8', 'ksc5601', 'iso-2022-kr' ],
+   'iso-2022-kr'   => [ 'utf-8', 'ksc5601', 'euc-kr' ],
    'ksc5601'       => [ 'utf-8', 'euc-kr', 'iso-2022-kr' ],
-   'gb2312'        => [ 'utf-8', 'big5', 'gbk' ],
-   'gbk'           => [ 'utf-8', 'big5', 'gb2312' ],
+   'gb2312'        => [ 'utf-8', 'big5' ],
    'iso-2022-jp'   => [ 'utf-8', 'shift_jis', 'euc-jp' ],
    'iso-8859-1'    => [ 'utf-8', 'windows-1252' ],
    'iso-8859-2'    => [ 'utf-8', 'windows-1250' ],
@@ -78,46 +87,118 @@ use vars qw(%charset_convlist %charset_localname %localname_cache);
    'windows-1255'  => [ 'utf-8', 'iso-8859-8' ],
    'windows-1256'  => [ 'utf-8', 'iso-8859-6' ],
    'windows-1257'  => [ 'utf-8', 'iso-8859-13' ],
-   'utf-8'         => [ 'big5', 'euc-jp', 'euc-kr', 'gb2312', 'gbk',
+   'utf-8'         => [ 'big5', 'euc-jp', 'euc-kr', 'gb2312',
 			'iso-2022-jp', 'iso-2022-kr',
 			'iso-8859-1', 'iso-8859-2', 'iso-8859-5', 'iso-8859-6',
 			'iso-8859-7', 'iso-8859-9', 'iso-8859-13',
-			'koi8-r', 'koi8-u', 'ksc5601', 'ks_c_5601-1987',
+			'koi8-r', 'koi8-u', 'ksc5601',
 			'shift_jis', 'tis-620',
 			'windows-1250', 'windows-1251', 'windows-1252',
 			'windows-1253', 'windows-1254', 'windows-1255',
 			'windows-1256', 'windows-1257' ]
    );
 
-sub is_convertable {
-   my ($from, $to)=@_;
-   $from=lc($from); $to=lc($to);
 
-   return 1 if ($from eq 'big5' && ($to eq 'gb2312'||$to eq 'gbk'));
-   return 1 if (($from eq 'gb2312'||$from eq 'gbk') && $to eq 'big5');
-   return 1 if ($from eq 'shift_jis' && $to eq 'iso-2022-jp');
-   return 1 if ($from eq 'iso-2022-jp' && $to eq 'shift_jis');
-   return 1 if ($from eq 'shift_jis' && $to eq 'euc-jp');
-   return 1 if ($from eq 'euc-jp' && $to eq 'shift_jis');
-
-   return 0 if ( !defined($charset_convlist{$to}) );
-   foreach my $charset (@{$charset_convlist{$to}}) {
-      if ($from=~/$charset/) {
-         my $converter=iconv_open($charset, $to);
-         return 0 if (! $converter);
-         $converter='';
-         return 1;
-      }
-   }
-   return 0;
+sub official_charset {
+   my $charset=lc($_[0]);
+   $charset=~s/iso_?8859/iso\-8859/;
+   $charset=$charset_equiv{$charset} if (defined $charset_equiv{$charset});
+   return $charset;
 }
 
-%localname_cache=();
+
+use vars qw(%is_convertible_cache);
+%is_convertible_cache=(
+   'big5#gb2312' => 1,
+   'gb2312#big5' => 1,
+   'shift_jis#isp-2022-jp' => 1,
+   'iso-2022-jp#euc-jp' => 1,
+   'euc-jp#shift_jis' => 1
+);
+sub is_convertible {
+   my ($from, $to)=@_;
+   return 0 if ($from eq'' || $to eq '');
+
+   $from=official_charset($from);
+   $to=official_charset($to);
+   return 0 if ($from eq $to || 			# not necessary
+                !defined $charset_convlist{$to} || 	# unrecognized to charset
+                !defined $charset_convlist{$from});	# unrecognized from charset
+   return 1 if ($from eq 'utf-8' || $to eq 'utf-8');	# utf8 is convertible with any charset
+
+   if (!defined $is_convertible_cache{"$from#$to"}) {
+      $is_convertible_cache{"$from#$to"}=0;
+      foreach my $charset (@{$charset_convlist{$to}}) {	# try all possible from charset
+         if ($from eq $charset) {
+            my $converter;
+            if ($converter=iconv_open($charset, $to)) {
+               $is_convertible_cache{"$from#$to"}=1;
+               $converter='';
+            }
+            last;
+         }
+      }
+   }
+   return $is_convertible_cache{"$from#$to"};
+}
+
+
+sub iconv {
+   my ($from, $to, @text)=@_;
+   return (@text) if (!is_convertible($from, $to));
+
+   $from=official_charset($from);
+   $to=official_charset($to);
+
+   for (my $i=0; $i<=$#text; $i++) {
+      next if ($text[$i]!~/[^\s]/);
+
+      # try convertion routine in iconv-chinese, iconv-japan first
+      if ($from  eq 'big5' && $to eq 'gb2312' ) {
+         $text[$i]=b2g($text[$i]); next;
+      } elsif ($from eq 'gb2312' && $to eq 'big5' ) {
+         $text[$i]=g2b($text[$i]); next;
+      } elsif ($from eq 'shift_jis' && $to eq 'iso-2022-jp' ) {
+         sjis2jis(\$text[$i]); next;
+      } elsif ($from eq 'iso-2022-jp' && $to eq 'shift_jis' ) {
+         jis2sjis(\$text[$i]); next;
+      } elsif ($from eq 'shift_jis' && $to eq 'euc-jp' ) {
+         sjis2euc(\$text[$i]); next;
+      } elsif ($from eq 'euc-jp' && $to eq 'shift_jis' ) {
+         euc2sjis(\$text[$i]); next;
+      } else {
+         $text[$i]=~s/(\S+)/_iconv($from, $to, $1)/egis;
+      }
+   }
+
+   return (@text);
+}
+# this routine try to keep opened iconv handle to speedup repeated conversion
+use vars qw($_iconv_handle $_iconv_tag);
+sub _iconv {
+   my ($from, $to, $s)=@_;
+   if ($_iconv_handle eq '' || $_iconv_tag ne "$from#$to" ) {
+      $_iconv_handle=iconv_open($from, $to);
+      $_iconv_tag = "$from#$to" if ($_iconv_handle ne '');
+   }
+   return $s if ($_iconv_handle eq '');		# no supported charset?
+
+   my $converted=$_iconv_handle->convert($s);
+   if ($converted ne '') {
+      return $converted;
+   } else {
+      $_iconv_handle='';   			# terminate converter
+      return "[".uc($from)."?]".$s;	# add [charset?] at the beginning if covert failed
+   }
+}
+
+
+use vars qw(%localname_cache); %localname_cache=();
 sub iconv_open {
    my ($from, $to)=@_;
 
-   if (defined($localname_cache{$from}) &&
-       defined($localname_cache{$to}) ) {
+   if (defined $localname_cache{$from} &&
+       defined $localname_cache{$to}) {
       return(Text::Iconv->new($localname_cache{$from}, $localname_cache{$to}));
    }
 
@@ -134,52 +215,4 @@ sub iconv_open {
    return('');
 }
 
-sub iconv {
-   my ($from, $to, @text)=@_;
-   $from=lc($from); $to=lc($to);
-
-   my $converter = iconv_open($from, $to);
-   my @result;
-
-   for (my $i=0; $i<=$#text; $i++) {
-      if ($text[$i]!~/[^\s]/) {
-         $result[$i]=$text[$i]; next;
-      }
-      # try convertion routine in iconv-chinese, iconv-japan first
-      if ($from  eq 'big5' && ($to eq 'gb2312'||$to eq 'gbk') ) {
-         $result[$i]=b2g($text[$i]); next;
-      } elsif (($from eq 'gb2312'||$from eq 'gbk') && $to eq 'big5' ) {
-         $result[$i]=g2b($text[$i]); next;
-      } elsif ($from eq 'shift_jis' && $to eq 'iso-2022-jp' ) {
-         $result[$i]=$text[$i]; sjis2jis(\$result[$i]); next;
-      } elsif ($from eq 'iso-2022-jp' && $to eq 'shift_jis' ) {
-         $result[$i]=$text[$i]; jis2sjis(\$result[$i]); next;
-      } elsif ($from eq 'shift_jis' && $to eq 'euc-jp' ) {
-         $result[$i]=$text[$i]; sjis2euc(\$result[$i]); next;
-      } elsif ($from eq 'euc-jp' && $to eq 'shift_jis' ) {
-         $result[$i]=$text[$i]; euc2sjis(\$result[$i]); next;
-      }
-      if ($converter) {
-         my @line=split(/\n/, $text[$i]);
-         for (my $j=0; $j<=$#line; $j++) {
-            next if ($line[$j]=~/^\s*$/);
-            my $converted=$converter->convert($line[$j]);
-            if ($converted ne '') {
-               $line[$j]=$converted;
-            } else {
-               # add [charset?] at the beginning if covert failed
-               $line[$j]="[".uc($from)."?]".$line[$j];
-               $converter='';	# free mem?
-               $converter = iconv_open($from, $to);
-            }
-         }
-         $result[$i]=join("\n", @line);
-      } else {
-         $result[$i]=$text[$i];
-      }
-   }
-   $converter='';
-   return (@result);
-}
-
 1;
