@@ -10,6 +10,7 @@ use Fcntl qw(:DEFAULT :flock);
 # extern vars, defined in caller openwebmail-xxx.pl
 use vars qw(%config %prefs %lang_text %lang_err);
 use vars qw($domain $user $uuid $homedir);
+use vars qw(@openwebmailrcitem);	# defined in ow-shared.pl
 
 sub upgrade_20030323 {		# called only if homedir doesn't exist
    # rename old homedir for compatibility
@@ -56,7 +57,7 @@ sub upgrade_all {	# called if user releasedate is too old
       if ( -f "$folderdir/.filter.book" ) {
          $content="";
          ow::filelock::lock("$folderdir/.filter.book", LOCK_EX) or
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdir/.filter.book");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} $folderdir/.filter.book");
          open(F, "$folderdir/.filter.book");
          while (<F>) {
             chomp;
@@ -82,7 +83,7 @@ sub upgrade_all {	# called if user releasedate is too old
       if ( -f "$folderdir/.pop3.book" ) {
          $content="";
          ow::filelock::lock("$folderdir/.pop3.book", LOCK_EX) or
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdir/.pop3.book");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} $folderdir/.pop3.book");
          open(F, "$folderdir/.pop3.book");
          while (<F>) {
             chomp;
@@ -117,7 +118,7 @@ sub upgrade_all {	# called if user releasedate is too old
          if ( -f "$folderdir/$book" ) {
             $content="";
             ow::filelock::lock("$folderdir/$book", LOCK_EX) or
-               openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdir/$book");
+               openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} $folderdir/$book");
             open(F, "$folderdir/$book");
             while (<F>) {
                last if (/\@\@\@/);
@@ -141,7 +142,7 @@ sub upgrade_all {	# called if user releasedate is too old
       my @cachefiles;
       my $file;
       opendir(FOLDERDIR, "$folderdir") or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderdir ($!)");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_read'} $folderdir ($!)");
       while (defined($file = readdir(FOLDERDIR))) {
          if ($file=~/^(\..+\.cache)$/) {
             $file="$folderdir/$1";
@@ -161,7 +162,7 @@ sub upgrade_all {	# called if user releasedate is too old
       if ( -f "$folderdir/.calendar.book" ) {
          my $content='';
          ow::filelock::lock("$folderdir/.calendar.book", LOCK_EX) or
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdir/.calendar.book");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} $folderdir/.calendar.book");
          open(F, "$folderdir/.calendar.book");
          while (<F>) {
             next if (/^#/);
@@ -207,7 +208,7 @@ sub upgrade_all {	# called if user releasedate is too old
       if ( -f "$folderdir/.pop3.book" ) {
          $content="";
          ow::filelock::lock("$folderdir/.pop3.book", LOCK_EX) or
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $folderdir/.pop3.book");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} $folderdir/.pop3.book");
          open(F, "$folderdir/.pop3.book");
          while (<F>) {
             chomp;
@@ -247,7 +248,7 @@ sub upgrade_all {	# called if user releasedate is too old
       )) { $is_dotpath{$_}=1; }
 
       opendir(FOLDERDIR, "$folderdir") or
-         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $folderdir ($!)");
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_read'} $folderdir ($!)");
       while (defined(my $file = readdir(FOLDERDIR))) {
          next if ($file eq '..' || $file!~/^\./);
          $file=~s/^\.//;
@@ -267,7 +268,7 @@ sub upgrade_all {	# called if user releasedate is too old
       if ( -f $pop3book ) {
          $content="";
          ow::filelock::lock($pop3book, LOCK_EX) or
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $pop3book");
+            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_writelock'} $pop3book");
          open(F, $pop3book);
          while (<F>) {
             chomp;
@@ -292,6 +293,73 @@ sub upgrade_all {	# called if user releasedate is too old
       }
    }
 
+   if ( $user_releasedate lt "20040724" ) {
+      my $filterbook = dotpath('filter.book');
+      my $filterruledb = dotpath('filter.ruledb');
+      # rename filter.book.db -> filter.ruledb.db
+      ow::dbm::rename($filterbook, $filterruledb);
+      writehistory("release upgrade - mv $filterbook to $filterruledb by 20040724");
+      writelog("release upgrade - mv $filterbook to $filterruledb by 20040724");
+   }
+
+   if ( $user_releasedate lt "20041101" ) {
+      my $rcfile=dotpath('openwebmailrc');
+      if (-f $rcfile) {
+         %prefs = readprefs();
+         $prefs{'abook_width'}=$config{'default_abook_width'};
+         $prefs{'abook_height'}=$config{'default_abook_height'};
+         $prefs{'abook_listviewfieldorder'}=$config{'default_abook_listviewfieldorder'};
+         if (open (RC, ">$rcfile")) {
+            foreach my $key (@openwebmailrcitem) {
+               print RC "$key=$prefs{$key}\n";
+            }
+            close (RC);
+         }
+         writehistory("release upgrade - openwebmailrc by 20041101");
+         writelog("release upgrade - openwebmailrc by 20041101");
+      }
+   }
+
+   if ( $user_releasedate lt "20041107" ) {
+      my $calbookfile=dotpath('calendar.book');
+      my $data;
+      if (open(F, $calbookfile)) {
+         while (<F>) {
+            chomp;
+            my @a=split(/\@\@\@/, $_);
+            $a[8]=$prefs{'charset'} if ($a[8] eq '');
+            $data.=join('@@@', @a)."\n";
+         }
+         close(F);
+         if (open(F, ">$calbookfile")) {
+            print F $data;
+            close(F);
+            writehistory("release upgrade - $calbookfile charset by 20041107");
+            writelog("release upgrade - $calbookfile charset by 20041107");
+         }
+      }
+   }
+
+   if ( $user_releasedate lt "20050206" ) {
+      my $filterbookfile=dotpath('filter.book');
+      my $data;
+      if (open(F, $filterbookfile)) {
+         while (<F>) {
+            chomp;
+            my @a=split(/\@\@\@/, $_);
+            $a[7]=$prefs{'charset'} if ($a[7] eq '');
+            $data.=join('@@@', @a)."\n";
+         }
+         close(F);
+         if (open(F, ">$filterbookfile")) {
+            print F $data;
+            close(F);
+            writehistory("release upgrade - $filterbookfile charset by 20050206");
+            writelog("release upgrade - $filterbookfile charset by 20050206");
+         }
+      }
+   }
+   return;
 }
 
 sub read_releasedatefile {
@@ -309,10 +377,32 @@ sub read_releasedatefile {
 
 sub update_releasedatefile {
    my $releasedatefile=dotpath('release.date');
-   open(D, ">$releasedatefile") or 
-      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_open'} $releasedatefile ($!)");
+   open(D, ">$releasedatefile") or
+      openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_write'} $releasedatefile ($!)");
    print D $config{'releasedate'};
    close(D);
 }
 
+sub update_openwebmailrc {
+   my $user_releasedate=$_[0];
+
+   my $rcfile=dotpath('openwebmailrc');
+   my $saverc=0;
+   if (-f $rcfile) {
+      $saverc=1 if ( $user_releasedate lt "20050220" );	# rc upgrade
+      %prefs = readprefs() if ($saverc);		# load user old prefs + sys defaults
+   } else {
+      $saverc=1 if ($config{'auto_createrc'});		# rc auto create
+   }
+   if ($saverc) {
+      open (RC, ">$rcfile") or
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_write'} $rcfile! ($!)");
+      foreach my $key (@openwebmailrcitem) {
+         print RC "$key=$prefs{$key}\n";
+      }
+      close (RC) or openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_close'} $rcfile!");
+   }
+   return;
+}
+
 1;
