@@ -19,9 +19,9 @@
             }
             if (!warn) {
                warn=1;
+               @@@JSCODE@@@
                hWnd = window.open(url,"_timeoutwarning","width=400,height=140,resizable=no,scrollbars=no");
                hWnd.focus();
-               @@@JSCODE@@@
             }
          }
       } else {
@@ -30,6 +30,15 @@
       }
    }
 
+   function sessioncheck () {
+      if (remainingseconds>0) {
+         return true;
+      } else {
+         alert("å󤬻ڤˤʤޤ\n١󤷤Ƥ");
+         return false;
+      }
+   }
+
    timeoutcheck();
 //-->
 </script>
