@@ -30,6 +30,15 @@
       }
    }
 
+   function sessioncheck () {
+      if (remainingseconds>0) {
+         return true;
+      } else {
+         alert("ܱǸ, ҵ׶Ѿyʱ, µ");
+         return false;
+      }
+   }
+
    timeoutcheck();
 //-->
 </script>
