@@ -30,6 +30,15 @@
       }
    }
 
+   function sessioncheck () {
+      if (remainingseconds>0) {
+         return true;
+      } else {
+         alert("ܩp, z@~qwgѮ, ЭsnJ");
+         return false;
+      }
+   }
+
    timeoutcheck();
 //-->
 </script>
